#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib.error
import urllib.parse
import urllib.request


def media_remote_action(data, action):
    if action == "toggleplay":
        player = get_active_player(data)

        if player is None:
            return "notplaying"

        command = {
            'jsonrpc': '2.0', 'method': 'Player.PlayPause', 'params': {'playerid': int(player['playerid'])}, 'id': int(1)
        }

        result = send_kodi_command(command, data)

        result = json.loads(result)

        #print result

        if result['result']['speed'] is 0:
            return "play"
        elif result['result']['speed'] is 1:
            return "pause"
        else:
            return "error"
    elif action == "up":
        method = "Input.Up"
    elif action == "down":
        method = "Input.Down"
    elif action == "left":
        method = "Input.Left"
    elif action == "right":
        method = "Input.Right"
    elif action == "select":
        method = "Input.select"
    elif action == "home":
        method = "Input.Home"
    elif action == "menu":
        method = "Input.ContextMenu"
    elif action == "volumedown":
        method = ""
    elif action == "volumeup":
        method = ""
    elif action == "togglemute":
        method = "Application.SetMute"
    else:
        return "nosuchaction"

    command = {
        'jsonrpc': '2.0', 'method': method, 'id': int(1)
    }

    return send_kodi_command(command, data)


def get_movies(data, limit, offset):
    command = {
        'jsonrpc': '2.0', 'method': 'VideoLibrary.GetMovies', 'params':{
            'properties':['thumbnail', 'genre', 'year'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
            'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libMovies'
    }

    return send_kodi_command(command, data)

def get_tv_shows(data, limit, offset):
    command = {
        'jsonrpc': '2.0', 'method': 'VideoLibrary.GetTVShows', 'params':{
            'properties':['thumbnail', 'genre', 'year'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
            'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libTvShows'
    }

    return send_kodi_command(command, data)

def get_music(data, limit, offset):
    command = {
        'jsonrpc': '2.0', 'method': 'AudioLibrary.GetSongs', 'params':{
            'properties':['artist', 'album', 'track'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
            'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libSongs'
    }

    return send_kodi_command(command, data)

def get_artists(data, limit, offset):
    command = {
        'jsonrpc': '2.0', 'method': 'AudioLibrary.GetArtists', 'params':{
            'properties':['thumgnail'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
            'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libArtists'
    }

    return send_kodi_command(command, data)

def get_music_genres(data, limit, offset):
    command = {
        'jsonrpc': '2.0', 'method': 'AudioLibrary.GetGenres', 'params':{
            'properties':['thubmnail', 'title'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
            'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libGenres'
    }

    return send_kodi_command(command, data)

def get_albums(data, limit, offset):
    command = {
        'jsonrpc': '2.0', 'method': 'AudioLibrary.GetAlbums', 'params':{
            'properties':['artist', 'title', 'thumbnail'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
            'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libAlbums'
    }

    return send_kodi_command(command, data)

def get_movie_genres(data, limit, offset):
    command = {
        'jsonrpc': '2.0', 'method': 'VideoLibrary.GetGenres', 'params':{'type':'movie',
            'properties':['thumbnail', 'title'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
            'sort':{'order': 'ascending', 'method': 'label'}},'id': 'libGenres'
    }

    return send_kodi_command(command, data)

def get_tv_show_seasons(data, show_id, limit, offset):
    command = {
        'jsonrpc': '2.0', 'method': 'VideoLibrary.GetSeasons', 'params':{
            'properties':['thumbnail'], 'limits':{'start': int(offset), 'end': int(offset)+int(limit)},
            'sort':{'order': 'ascending', 'method': 'label', 'ignorearticel': True}},'id': 'libSeasons', 'tvshowid': show_id
    }

    return send_kodi_command(command, data)

def send_text(data, text):
    command = {
        'jsonrpc': '2.0', 'method': 'Input.SendText', 'params': {'text': text, 'done': True}, 'id': 1
    }

    return send_kodi_command(command, data)

def get_active_player(data):
    command = {
        'jsonrpc': '2.0', 'method': 'Player.GetActivePlayers', 'id': 1
    }

    result = send_kodi_command(command, data)

    result = json.loads(result)

    #print result

    for player in result['result']:
        #print player
        return player

    return None

#def remote_action(data, action):

def send_kodi_command(command, data):

    command = json.dumps(command)

    #print command

    #command = '{"jsonrpc": "2.0","method": "VideoLibrary.GetMovies","params": {},"id": "libMovies"}'
    #print command

    ip = data['IP']

    port = data['PORT']

    if port is not "" and port is not None:
        port = ":" + port
    else:
        port = ""

    link = "http://"+ip+port+"/jsonrpc?request="+urllib.parse.quote(command)

    #print link

    result = urllib.request.urlopen(link).read()

    return result


#!/usr/bin/python
# -*- coding: utf-8 -*-

from Homevee.Utils.Database_NEW import Database

def get_toggle_devices(username, room, db: Database = None):
    if db is None:
        db = Database()
    devices = []
    results = db.select_all("SELECT * FROM URL_TOGGLE WHERE LOCATION = :location",
                {'location': room})

    for toggle in results:
        devices.append({'name': toggle['NAME'], 'id': toggle['ID'], 'type': 'URL-Toggle',
            'icon': toggle['ICON']})

    return {'toggles': devices}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.DeviceAPI import zwave
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *

def get_dimmers(user, room, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(room):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    dimmer = []
    #Z-Wave Dimmer
    results = db.select_all("SELECT * FROM ZWAVE_DIMMER WHERE LOCATION == :room", {'room': room})

    for item in results:
        dimmer_item = {'device': item['ID'], 'value': item['VALUE'], 'icon': item['ICON'],
                     'name': item['NAME'], 'type': ZWAVE_DIMMER}
        dimmer.append(dimmer_item)

    return {'dimmer': dimmer}

def set_dimmer(user, type, id, value, db: Database = None):
    if db is None:
        db = Database()
    if type == ZWAVE_DIMMER:
        return set_zwave_dimmer(user, id, value, db)
    else:
        raise ValueError("Type does not exist")

def set_zwave_dimmer(user, id, value, db: Database = None):
    if db is None:
        db = Database()

        data = db.select_one("SELECT * FROM ZWAVE_DIMMER WHERE ID == :device",
                    {'device': id})

        if not user.has_permission(data['LOCATION']):
            return {'result': 'nopermission'}

        device_id = data['ID']

        result = zwave.device_control.set_multistate_device(device_id, value, db)

        if result['code'] == 200:
            db.update("UPDATE ZWAVE_DIMMER SET VALUE = :value WHERE ID = :id",
                        {'value': value, 'id': id})

            return {'result':'ok'}


#!/usr/bin/python
# -*- coding: utf-8 -*-

import datetime
import json

from Homevee.Utils.Database_NEW import Database

'''Gibt das Wetter für die angegebene Anzahl an Tagen zurück'''
def get_weather(daycount, db: Database = None):
    if db is None:
        db = Database()
    if daycount > 16:
        daycount = 16

    weather_data = json.loads(db.get_server_data("WEATHER_CACHE"))

    days = weather_data['list']

    output = []

    for i in range(0, min(len(days), daycount)):
        day = days[i]

        relative_days = ["Heute", "Morgen", "Übermorgen"]
        day_names = ["Mo", "Di", "Mi", "Do", "Fr", "Sa", "So"]

        if i > 2:
            date = datetime.datetime.fromtimestamp(day['dt'])

            week_day_number = date.weekday()

            date = day_names[week_day_number]+", "+date.strftime('%d.%m.')
        else:
            date = relative_days[i]

        city_name = weather_data['city']['name']

        icon_link = "http://openweathermap.org/img/w/"+day['weather'][0]['icon']+".png"

        weather_desc = day['weather'][0]['description']
        icon = day['weather'][0]['icon']

        pressure = None
        if "pressure" in day:
            pressure = day['pressure']

        humidity = None
        if "humidity" in day:
            humidity = day['humidity']

        wind_speed = None
        if "speed" in day:
            wind_speed = float(day['speed'])*3.6

        wind_direction = None
        if "deg" in day:
            wind_direction = day['deg']

        clouds = None
        if "clouds" in day:
            clouds = day['clouds']

        rain = None
        if "rain" in day:
            rain = day['rain']

        snow = None
        if "snow" in day:
            snow = day['snow']

        day_item = {'city':city_name, 'date':date, 'temps':day['temp'], 'icon':icon_link, 'iconid':icon,
                    'desc':weather_desc, 'pressure':pressure, 'humidity':humidity, 'windspeed':wind_speed,
                    'winddirection': wind_direction, 'clouds':clouds, 'rain':rain, 'snow':snow}

        output.append(day_item)

    return output

'''Setzt die Orts-ID zur Wetter-Abfrage'''
def set_weather_city_id(request, db: Database = None):
    if db is None:
        db = Database()
    return

'''Gibt alle Orte zur Wetterabfrage zurück'''
def get_weather_city_list(db):
    try:
        return db.get_server_data("WEATHER_CITY_CACHE")
    except:
        return None


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.DeviceAPI import philips_hue, blind_control
from Homevee.DeviceAPI.heating import heating_control
from Homevee.DeviceAPI.set_modes import set_modes
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *
from Homevee.VoiceAssistant.Helper import generate_string, set_context, get_okay
from Homevee.VoiceAssistant.Modules.DeviceControlModule import VoiceDeviceControlModule

CONTEXT_SET_MODES = "CONTEXT_SET_MODES"
CONTEXT_SET_HEATING = "CONTEXT_SET_HEATING"
CONTEXT_SET_THERMOSTAT = "CONTEXT_SET_THERMOSTAT"
CONTEXT_CONTROL_BLINDS = "CONTEXT_CONTROL_BLINDS"

class VoiceDeviceSetModesModule(VoiceDeviceControlModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        data = [
            ['FUNKSTECKDOSEN', 'NAME', 'DEVICE', FUNKSTECKDOSE],
            ['ZWAVE_SWITCHES', 'NAME', 'ID', ZWAVE_SWITCH],
            ['URL_SWITCH_BINARY', 'NAME', 'ID', URL_SWITCH],
            ['URL_TOGGLE', 'NAME', 'ID', URL_TOGGLE],
            ['PHILIPS_HUE_LIGHTS', 'NAME', 'ID', PHILIPS_HUE_LIGHT],
            ['URL_RGB_LIGHT', 'NAME', 'ID', URL_RGB_LIGHT]
        ]
        device_names = []
        for item in data:
            results = db.select_all("SELECT * FROM " + item[0], {})
            for device in results:
                device_names.append(device[item[1]].lower())
        output_array = [
            [['mach', 'schalt'], device_names, ['an', 'ein', 'aus']],
            [device_names, ['an', 'ein', 'aus']]
        ]
        # print output_array
        return output_array

    def get_label(self):
        return "switch"

    def get_context_key(self):
        return [
            CONTEXT_SET_THERMOSTAT,
            CONTEXT_CONTROL_BLINDS,
            CONTEXT_SET_HEATING,
            CONTEXT_SET_MODES
        ]

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.set_modes_voice(username, text, context, db)

    def set_modes_voice(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        room = self.find_room(text, db)

        if room is not None:
            room_key = room['LOCATION']
        else:
            room_key = None

        device_types = [FUNKSTECKDOSE, ZWAVE_SWITCH, URL_SWITCH, URL_TOGGLE, PHILIPS_HUE_LIGHT, URL_RGB_LIGHT]

        devices = self.find_devices(text, device_types, room_key, db)

        if len(devices) == 0:
            #Keine Geräte gefunden
            answer_data = [
                [['Dieses ', 'Das genannte '], 'Gerät ', ['existiert nicht.', 'gibt es nicht.', 'wurde noch nicht angelegt.']]
            ]

            answer = generate_string(answer_data)

            return {'msg_speech': answer, 'msg_text': answer}

        words = text.split(" ")
        mode = None
        on_words = ['an', 'ein']
        off_words = ['aus']
        for word in words:
            if word in on_words:
                mode = 1
                break
            elif word in off_words:
                mode = 0
                break

        if mode is None:
            if 'mode' in context:
                mode = context['mode']
            else:
                # Kein Zustand angegeben
                answer_data = [
                    ['Soll ', 'ich ', 'die Geräte ', ['an', 'ein'], '- oder aus', ['schalten', 'machen'], [', '+username, ''], '?']
                ]

                answer = generate_string(answer_data)

                return {'msg_speech': answer, 'msg_text': answer}

        #Geräte schalten
        for device in devices:
            if(device['type'] == PHILIPS_HUE_LIGHT):
                philips_hue.set_light_mode(username, device['id'], mode, None, None, None, db)
            else:
                set_modes(username, device['type'], device['id'], mode, db)

        set_context(username, CONTEXT_SET_MODES, {'location': room, 'devices': devices, 'mode':mode}, db)

        MODE_WORDS = on_words
        if mode == 0:
            MODE_WORDS = off_words

        DEVICE_STRING = None
        if len(devices) > 1:
            VERB = "wurden"
            DEVICE_WORD = "Die Geräte "

            for i in range(0, len(devices)):
                if DEVICE_STRING is None:
                    DEVICE_STRING = '' + devices[i]['name'] + ''
                elif i == len(devices) - 1:
                    DEVICE_STRING = DEVICE_STRING + ' und ' + '' + devices[i]['name'] + ''
                else:
                    DEVICE_STRING = DEVICE_STRING + ', ' + '' + devices[i]['name'] + ''

        else:
            DEVICE_WORD = "Das Gerät "
            VERB = "wurde"

            DEVICE_STRING = '' + devices[0]['name'] + ''

        answer_data = [
            [get_okay(), [', ' + username, ''], '.',
             ['', [' ', DEVICE_WORD,  [DEVICE_STRING + ' ', ''], VERB + ' ',
                   MODE_WORDS, ['geschaltet', 'geschalten'], '.']]]
        ]

        answer = generate_string(answer_data)

        return {'msg_speech': answer, 'msg_text': answer}

    def control_blinds(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        words = text.split(" ")

        value = None

        if context is not None:
            if 'value' in context:
                value = int(context['value'])

        if value is None:
            if 'auf' not in words or len(words) <= words.index('auf') + 1:
                close_words = ['zu', 'runter', 'herunter']
                open_words = ['auf', 'rauf', 'hoch']

                for word in words:
                    if word in close_words:
                        value = 100
                        break
                    elif word in open_words:
                        value = 0
                        break
            else:
                value = words[words.index('auf') + 1]

        room = self.find_room(text, db)
        if room is not None:
            room_key = room['LOCATION']
        else:
            if context is not None and 'room' in context and context['room'] is not None:
                room = context['room']
                room_key = context['room']['LOCATION']
            else:
                # no room => control in every room
                room = None
                '''
                answer_data = [
                    [['Wo ', 'In welchem Raum '], 'soll ich das machen', [', ' + username, ''], '?']
                ]
    
                if value is not None:
                    set_context(username, CONTROL_BLINDS, {'value': value}, db)
    
                answer = generate_string(answer_data)
    
                return {'msg_speech': answer, 'msg_text': answer}
                '''

        #print(str(room) + ": " + str(value))

        blinds = []

        blinds_db_data = [
            {'table': 'HOMEPILOT_BLIND_CONTROL', 'room_col': 'LOCATION', 'id_col': 'ID', 'type': RADEMACHER_BLIND_CONTROL}
        ]

        for data_item in blinds_db_data:
            if room is not None:
                results = db.select_all('SELECT * FROM ' + data_item['table'] + ' WHERE ' + data_item['room_col'] + ' = :room',
                        {'room': room_key})
            else:
                results = db.select_all('SELECT * FROM ' + data_item['table'], db)

            for item in results:
                blinds_item = {'type': data_item['type'], 'id': item[data_item['id_col']]}
                blinds.append(blinds_item)

        #print(blinds)

        for blinds_item in blinds:
            blind_control.set_blinds(username, blinds_item['type'], blinds_item['id'], value, db)

        # set_context(username, SET_THERMOSTAT, {'room': room, 'value': value}, db)

        bezeichner = ['Der Rollladen wurde', 'Die Rolläden wurden', 'Die Jalousien wurden']

        if room is None:
            bezeichner = ['Alle Rolläden wurden', 'Alle Jalousien wurden', 'Jeder Rollladen wurde']

        opened_words = ['geöffnet', 'hochgefahren']
        closed_words = ['geschlossen', 'runtergefahren']

        value_string = ['auf '+str(value)+' % ', ['gefahren', 'eingestellt']]

        if value is 0:
            action_string = opened_words
        elif value is 100:
            action_string = closed_words
        else:
            action_string = value_string

        answer_data = [
            [get_okay(), [', ' + username, ''], '.',
             ['', [' ', bezeichner, ' ', action_string, '.']]]
        ]

        answer = generate_string(answer_data)

        return {'msg_speech': answer, 'msg_text': answer}

    def set_thermostat(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        words = text.split(" ")

        if 'auf' not in words or len(words) < words.index('auf')+1:
            if context is not None and 'value' in context and context['value'] is not None:
                value = context['value']
            else:
                answer_data = [
                    [['Welchen Wert ', 'Welche Temperatur'], ' soll ich einstellen', [', ' + username, ''], '?']
                ]

                set_context(username, CONTEXT_SET_THERMOSTAT, context, db)

                answer = generate_string(answer_data)

                return {'msg_speech': answer, 'msg_text': answer}
        else:
            value = words[words.index('auf')+1]

        room = self.find_room(text, db)
        if room is not None:
            room_key = room['LOCATION']
        else:
            if context is not None and 'room' in context and context['room'] is not None:
                room = context['room']
                room_key = context['room']['LOCATION']
            else:
                # no room
                answer_data = [
                    [['Wo ', 'In welchem Raum '], 'soll ich das machen', [', ' + username, ''], '?']
                ]

                if value is not None:
                    set_context(username, CONTEXT_SET_THERMOSTAT, {'value': value}, db)

                answer = generate_string(answer_data)

                return {'msg_speech': answer, 'msg_text': answer}

        #print(str(room)+": "+value)

        thermostats = []

        thermostat_db_data = [
            {'table': 'MAX_THERMOSTATS', 'room_col': 'RAUM', 'id_col': 'ID', 'type': MAX_THERMOSTAT},
            {'table': 'ZWAVE_THERMOSTATS', 'room_col': 'RAUM', 'id_col': 'THERMOSTAT_ID', 'type': ZWAVE_THERMOSTAT}
        ]

        for data_item in thermostat_db_data:
            results = db.select_all('SELECT * FROM '+data_item['table']+' WHERE '+data_item['room_col']+' = :room',
                        {'room': room_key})

            for item in results:
                thermostat = {'type': data_item['type'], 'id': item[data_item['id_col']]}
                thermostats.append(thermostat)

        #print(thermostats)

        for thermostat in thermostats:
            heating_control(username, thermostat['type'], thermostat['id'], value, db)

        #set_context(username, SET_THERMOSTAT, {'room': room, 'value': value}, db)

        answer_data = [
            [get_okay(), [', '+username, ''], '.',
             ['', [' ', ['Die Heizung', 'Das Thermostat', 'Die Temperatur'], ' wurde ',
              ['', ['auf '+ value + ' Grad ']],
              ['angepasst', 'neu eingestellt', 'geändert'], '.']]]
        ]

        answer = generate_string(answer_data)

        return {'msg_speech': answer, 'msg_text': answer}

    def set_dimmer(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        room = self.find_room(text, db)

        if room is not None:
            room_key = room['LOCATION']
        else:
            room_key = None

        device_types = [FUNKSTECKDOSE, ZWAVE_SWITCH, URL_SWITCH, URL_TOGGLE, PHILIPS_HUE_LIGHT, URL_RGB_LIGHT]

        devices = self.find_devices(text, device_types, room_key, db)
        return {'msg_speech':"Ok", 'msg_text':"Dimmer"}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Utils.Database_NEW import Database


def handle_mqtt_sensor(device_id, value, db: Database = None):
    if db is None:
        db = Database()

    item = db.select_one("SELECT * FROM MQTT_SENSORS WHERE ID = :id", {'id': device_id})

	#if(item['LAST_VALUE'] != value):
		#trigger automation

    db.update("UPDATE MQTT_SENSORS SET LAST_VALUE = :val WHERE ID = :id",
    {'val': value, 'id': device_id})

    #check if device is used in automation rules



#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import os
import socket
import subprocess
import time
import traceback
import urllib.error
import urllib.error
import urllib.parse
import urllib.parse
import urllib.request
import urllib.request
from socket import gethostname

from OpenSSL import crypto

from Homevee.Helper import Logger
from Homevee.Helper.HomeveeCloud import HomeveeCloudWrapper
from Homevee.Item.Status import *
from Homevee.Item.User import User
from Homevee.Utils import Constants
from Homevee.Utils.Database_NEW import Database


def save_request_to_db(data, reply, db: Database = None):
    if db is None:
        db = Database()
    return
    if 'password' in data:
        del data['password']

    if data['action'] == "arcontrol":
        del data['imagedata']

        db.insert("INSERT INTO REQUEST_DATA (RESPONSE) VALUES (:response)",
            {'response': reply})

        request_id = cur.lastrowid

        for key in list(data.keys()):
            db.insert("INSERT INTO REQUEST_DATA_PARAMS (REQUEST_ID, PARAM_KEY, PARAM_VALUE) VALUES(:id, :key, :value)",
                {'id': request_id, 'key': key, 'value': data[key]})

        return True

def send_to_client(data, conn, is_http):
    if not is_http:
        Logger.log(("Sent Response: "+data))
    elif is_http:
        data = 'HTTP/1.1 200 OK\nContent-Type: text/html\n'+data+'\n'
        Logger.log(("Sent HTTP-Response: "+data))

    #Prüfen, ob alle Daten gesendet wurden
    len_send = conn.send(bytearray(str.encode(data)))
    #len_send = conn.send(compressed_data)
    Logger.log(("Data: "+str(len(data))+" | Sent: "+str(len_send)))

    #if(len_send is 0):
    #    send_to_client(json.dumps({'status': 'error'}), conn, is_http)

def verify_user(username, password):
    db = Database()

    user = User.load_username_from_db(username, db)

    return user.verify(password)

def remote_control_enabled(db):
    REMOTE_CONTROL_ENABLED = db.get_server_data("USE_REMOTE_CONTROL", db)
    return REMOTE_CONTROL_ENABLED == "true"

def set_remote_control_enabled(user, enabled, db: Database = None):
    if db is None:
        db = Database()
    if(not user.has_permission("admin")):
        return {'result': 'noadmin'}
    db.set_server_data("USE_REMOTE_CONTROL", enabled, db)
    return Status(type=STATUS_OK).get_dict()

def save_remote_data(user, remote_id, linked_account, db: Database = None):
    if db is None:
        db = Database()
    if(not user.has_permission("admin")):
        return {'result': 'noadmin'}

    db.set_server_data("REMOTE_ID", remote_id, db)
    db.set_server_data("REMOTE_LINKED_ACCOUNT", linked_account, db)

    return Status(type=STATUS_OK).get_dict()

def load_remote_data(user, db: Database = None):
    if db is None:
        db = Database()
    if(not user.has_permission("admin")):
        return {'result': 'noadmin'}

    remote_id = db.get_server_data("REMOTE_ID", db)
    access_token = db.get_server_data("REMOTE_ACCESS_TOKEN", db)
    linked_account = db.get_server_data("REMOTE_LINKED_ACCOUNT", db)

    #check if account is still linked

    #load premium membership
    url = "https://cloud.homevee.de/server-api.php?action=getpremiumuntil&remoteid="+remote_id+"&accesstoken="+access_token
    contents = urllib.request.urlopen(url).read()
    data = json.loads(contents)
    is_premium = None

    try:
        is_premium = data['is_premium']
        premium_until = data['premium_until']
    except:
        premium_until = None

    return {'remote_id': remote_id,
            'access_token': access_token,
            'linked_account': linked_account, 'premium_until': premium_until, 'is_premium': is_premium,
            'remote_control_enabled': remote_control_enabled(db)}

def connect_remote_id_with_account(user, account_name, account_secret, db: Database = None):
    if db is None:
        db = Database()
    if(not user.has_permission("admin")):
        return {'result': 'noadmin'}

    remote_id = db.get_server_data("REMOTE_ID", db)
    url = "https://Homevee.de/connect-remote-id.php?remote_id="+remote_id+"&account_name="\
          +account_name+"&account_secret="+account_secret
    response = urllib.request.urlopen(url).read()

    response = response.decode('utf-8')

    if response == "ok":
        db.set_server_data("REMOTE_LINKED_ACCOUNT", account_name, db)

    return {'result': response}

def update_ip_thread():
    last_ip = None

    db = Database()

    while(True):
        remote_id = db.get_server_data("REMOTE_ID")
        access_token = db.get_server_data("REMOTE_ACCESS_TOKEN")
        my_ip = get_my_ip()

        if my_ip != last_ip or my_ip != get_my_ip_from_cloud(remote_id):
            try:
                update_ip(my_ip, remote_id, access_token)
            except:
                pass

        last_ip = my_ip

        #wait some time
        time.sleep(5*60) #5 Minuten

def update_cert_thread():
    db = Database()

    while (True):
        remote_id = db.get_server_data("REMOTE_ID")
        access_token = db.get_server_data("REMOTE_ACCESS_TOKEN")

        # generate_cert

        check_cert(None, remote_id, access_token)

        # wait some time
        time.sleep(12 *60 * 60)  # 12 Stunden

def check_cert(db=None, remote_id=None, access_token=None):
    cert_data = get_local_cert()

    if db is not None:
        remote_id = db.get_server_data("REMOTE_ID")
        access_token = db.get_server_data("REMOTE_ACCESS_TOKEN")

    if cert_data is None:
        generate_cert()

    if cert_data is not None and cert_data != get_my_cert_from_cloud(remote_id):
        update_cert(cert_data, remote_id, access_token)

def update_ip(ip, remote_id, access_token):

    MAX_RETRIES = 10
    retries = 0

    contents = None

    # update local ip in cloud
    homevee_cloud = HomeveeCloudWrapper(remote_id, access_token)
    homevee_cloud.set_cert(ip)
    '''
    try:
        url = "https://cloud.homevee.de/server-api.php?action=updatelocalip&remoteid=" + remote_id + "&accesstoken=" + access_token + "&localip=" + ip
        #Logger.log(remote_id+" - "+access_token+" - "+ip)
        while (contents != "ok" and retries < MAX_RETRIES):
            try:
                contents = urllib.request.urlopen(url).read()
                return True
            except:
                #if(Logger.IS_DEBUG):
                traceback.print_exc()
                Logger.log(translations.translate("no_cloud_connection"))
            retries += 1
        return False
    except:
        return False
    '''

def update_cert(cert_data, remote_id, access_token):
    Logger.log("updating local cert...")

    cert_data = cert_data.replace("\n", "")
    cert_data = cert_data.replace("-----BEGIN CERTIFICATE-----", "")
    cert_data = cert_data.replace("-----END CERTIFICATE-----", "")

    MAX_RETRIES = 10
    retries = 0
    contents = None

    #print(remote_id, access_token, cert_data)

    # update cert in cloud
    try:
        homevee_cloud = HomeveeCloudWrapper(remote_id, access_token)
        homevee_cloud.set_cert(cert_data)
    except:
        Logger.log("Could not update local cert...")

    '''
    try:
        url = "https://cloud.homevee.de/server-api.php?action=updatecert&remoteid=" + remote_id + "&accesstoken=" + access_token + "&cert=" + urllib.parse.quote(
        cert_data)
        #print(url)

        while (contents != "ok" and retries < MAX_RETRIES):
            try:
                contents = urllib.request.urlopen(url).read()
                Logger.log(contents)
                return True
            except:
                if(Logger.IS_DEBUG):
                    traceback.print_exc()
            retries += 1
        return False
    except:
        return False
    '''

def get_my_ip():
    if Database().get_server_data("REMOTE_ID") == "VX6FLAYZN":
        return "192.168.2.119"

    try:
        cmd = "hostname -i"
        data = subprocess.check_output(cmd, shell=True).decode('utf-8')
        #return "192.168.2.110"
        ip, mac = data.split(" ")
    except:
        traceback.print_exc()
        ip = socket.gethostbyname(socket.gethostname())

    Logger.log("my ip address: " + ip)

    return ip

def generate_cert():
    # create a key pair
    k = crypto.PKey()
    k.generate_key(crypto.TYPE_RSA, 1024)

    # create a self-signed cert
    cert = crypto.X509()
    cert.get_subject().C = "DE"
    cert.get_subject().ST = "Germany"
    cert.get_subject().L = "Germany"
    cert.get_subject().O = "Homevee"
    cert.get_subject().OU = "Homevee"
    cert.get_subject().CN = gethostname()
    cert.set_serial_number(1000)
    cert.gmtime_adj_notBefore(0)
    cert.gmtime_adj_notAfter(10 * 365 * 24 * 60 * 60)
    cert.set_issuer(cert.get_subject())
    cert.set_pubkey(k)
    cert.sign(k, 'sha1')

    open(Constants.LOCAL_SSL_CERT, "wt").write(
        crypto.dump_certificate(crypto.FILETYPE_PEM, cert).decode('utf-8'))
    open(Constants.LOCAL_SSL_PRIVKEY, "wt").write(
        crypto.dump_privatekey(crypto.FILETYPE_PEM, k).decode('utf-8'))

def get_local_cert():
    if(os.path.isfile(Constants.LOCAL_SSL_CERT)):
        file = open(Constants.LOCAL_SSL_CERT, "r")
        content = file.read()
        if content is not None and content != '':
            return content

    return None

def get_my_ip_from_cloud(remote_id):
    '''
    try:
        url = "http://cloud.homevee.de/server-api.php?action=getlocalip&remoteid="+remote_id
        #print(url)
        contents = urllib.request.urlopen(url).read()

        data = json.loads(contents)

        if 'ip' in data and data['ip'] is not None:
            return data['ip']
        else:
            return None
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()
        return None
    '''
    homevee_cloud = HomeveeCloudWrapper(remote_id, Database().get_server_data("REMOTE_ACCESS_TOKEN"))
    try:
        return homevee_cloud.get_ip()
    except:
        return None

def get_my_cert_from_cloud(remote_id):
    '''
    try:
        url = "http://cloud.homevee.de/server-api.php?action=getcert&remoteid="+remote_id
        contents = urllib.request.urlopen(url).read()

        data = json.loads(contents)

        if 'cert' in data and data['cert'] is not None:
            return data['cert']
        else:
            return None
    except:
        if(Logger.IS_DEBUG):
            traceback.print_exc()
        return None
    '''
    homevee_cloud = HomeveeCloudWrapper(remote_id, Database().get_server_data("REMOTE_ACCESS_TOKEN"))
    try:
        return homevee_cloud.get_cert()
    except:
        return None



#!/usr/bin/python
# -*- coding: utf-8 -*-
import base64
import hashlib
from random import Random

from cryptography.hazmat.primitives.ciphers.algorithms import AES


class AESCipher(object):

    def __init__(self, key):
        self.bs = 32
        self.key = hashlib.sha256(key.encode()).digest()

    def encrypt(self, raw: str) -> str:
        """
        Encrypts the given string
        :param raw: the raw data string
        :return: the encrypted string
        """
        raw = self._pad(raw)
        iv = Random.new().read(AES.block_size)
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return base64.b64encode(iv + cipher.encrypt(raw))

    def decrypt(self, enc: str) -> str:
        """
        Decrypts the given data string
        :param enc: the encrypted string
        :return: the decrypted string
        """
        enc = base64.b64decode(enc)
        iv = enc[:AES.block_size]
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return self._unpad(cipher.decrypt(enc[AES.block_size:])).decode('utf-8')

    def _pad(self, s):
        return s + (self.bs - len(s) % self.bs) * chr(self.bs - len(s) % self.bs)

    @staticmethod
    def _unpad(s):
        return s[:-ord(s[len(s)-1:])]


import json
import time
import traceback

from Homevee.Exception import InvalidParametersException, DatabaseSaveFailedException
from Homevee.Helper import Logger
from Homevee.Item import Item
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.NotificationManager import NotificationManager


class ChatMessage(Item):
    def __init__(self, username, data, timestamp=None, id=None):
        super(ChatMessage, self).__init__()
        self.username = username
        self.data = data
        self.timestamp = timestamp
        self.id = id
        #self.format_time()

    def send(self, db: Database = None):
        self.save_to_db()
        self.timestamp = time.time()

        results = db.select_all("SELECT USERNAME FROM USERDATA WHERE USERNAME != :user", {'user': self.username})
        users = []

        for user in results:
            users.append(user['USERNAME'])

        NotificationManager().send_notification_to_users(users, "Neue Nachricht von " + self.username,
                                                       json.loads(self.data)['msg'], db,
                                                       click_action="ChatActivity")

        return {'username': self.username, 'data': self.data, 'time': self.timestamp}


    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            if (self.id is None or self.id == ""):
                db.insert("""INSERT INTO CHAT_DATA (USERNAME, DATA) VALUES (:username, :data, :timestamp)""",
                            {'username': self.username, 'data': self.data, 'timestamp': time.time()})
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save event to database")

    def build_dict(self):
        dict = {
            'id': self.id,
            'timestamp': self.timestamp,
            #'formatted_timestamp': self.formatted_timestamp,
            'data': self.data,
            'username': self.username
        }
        return dict

    '''
    def format_time(self):
        try:
            if self.timestamp is not None:
                local_time = datetime.datetime.fromtimestamp(int(self.timestamp))
                formatted_timestamp = local_time.strftime("%d.%m.%Y %H:%M")
                self.formatted_timestamp = formatted_timestamp
        except:
            self.formatted_timestamp = self.timestamp
    '''

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = ChatMessage(result['USERNAME'], result['DATA'], result['TIMESTAMP'], result['ID'])
            items.append(item)
        return items

    @staticmethod
    def load_all(db: Database):
        return ChatMessage.load_all_from_db('SELECT * FROM CHAT_DATA', {}, db)

    @staticmethod
    def load_all_by_time(time: int, limit: int, db: Database = None) -> list:
        """
        Loads all (or limited) chat messages before a timestamp
        :param time: the time in unixtime format
        :param limit: the number of results to show
        :param db: the database connection
        :return: the list of ChatMessages
        """
        where_clause = ""
        params = {'limit': limit}

        if time is not None and int(time) is not -1:
            where_clause = "WHERE TIMESTAMP < :time"
            params['time'] = time

        return ChatMessage.load_all_from_db("SELECT * FROM CHAT_DATA "+where_clause+" ORDER BY TIMESTAMP DESC LIMIT :limit",
        params, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            timestamp = dict['timestamp']
            data = dict['data']
            username = dict['username']

            item = ChatMessage(username, data, timestamp, id)

            return item
        except:
            raise InvalidParametersException("ChatMessage.create_from_dict(): invalid dict")


import json
import os
import urllib
from _thread import start_new_thread

import pip
from packaging import version

from Homevee.Item import User
from Homevee.Item.Status import *
from Homevee.Utils import Constants
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.NotificationManager import NotificationManager


def get_homevee_update_version():
    """
    Gets the updateable homevee-version
    :return: None if no newer version found, version number otherwise
    """
    installed_version = Constants.HOMEVEE_VERSION_NUMBER

    newest_version = get_newest_version()

    if(newest_version is None):
        return None

    if(version.parse(newest_version) > version.parse(installed_version)):
        return newest_version
    else:
        return None

def get_newest_version() -> str:
    """
    Gets the newest homevee-version from pypi
    :return: the version string or None
    """
    url = "https://pypi.org/pypi/Homevee/json"

    try:
        response = urllib.request.urlopen(url).read()
        response = response.decode('utf-8')
        response_json = json.loads(response)
        version = response_json['info']['version']

        return version
    except:
        return None

def check_for_updates() -> dict:
    """
    Checks if a new homevee-version is available on pypi
    :return:
    """
    new_version = get_homevee_update_version()

    return {
        'updates':{
            'current_version': Constants.HOMEVEE_VERSION_NUMBER,
            'new_version': new_version,
            'update_available': (new_version is not None),
            'changelog': "Changelog blabla..." #TODO add changelog or link to actual changelog
        }
    }

'''
Updates the Homevee PIP-Package
Returns true if update was successful,
returns false if there was an error
'''
def do_homevee_update(user: User, db: Database) -> dict:
    """
    Starts the homevee update-process
    :param user: the user
    :param db: the database connection
    :return: the status of the update process
    """
    if(not user.has_permission("admin")):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    start_new_thread(update_thread, ())

    return Status(type=STATUS_OK).get_dict()

def update_thread():
    """
    The thread for the update process of homevee
    :return:
    """
    new_version = get_homevee_update_version()

    try:
        pip.main(["install", "--upgrade", "Homevee"])
    except:
        return False

    #Datenbank upgraden
    Database().upgrade()

    # TODO texte lokalisieren
    title = "Update"
    body = "Update auf Version " + new_version

    # Send notification to admin
    NotificationManager().send_notification_to_admin(title, body, Database())

    # Reboot the system after the update
    os.system('reboot')


class ItemNotFoundException(Exception):
    '''Raised when an item was not found'''
    pass

class InvalidParametersException(Exception):
    '''Raised when parameters where invalid'''
    pass

class DatabaseSaveFailedException(Exception):
    '''Raised when saving to database fails'''
    pass

class RoomHasDataException(Exception):
    '''Raised when a room that should be deleted has data left'''
    pass

class AbstractFunctionCallException(Exception):
    '''Raised when a abstract method is called'''
    pass

class NoSuchTypeException(Exception):
    '''Raised when a devicetype is used that does not exist'''
    pass


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.DeviceAPI.zwave.device_control import get_data
from Homevee.DeviceAPI.zwave.utils import do_zwave_request
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *


def get_devices(db: Database):
    if db is None:
        db = Database()
    json_data = do_zwave_request("/ZAutomation/api/v1/locations", db)

    room_data = json_data['data']

    devices = []

    for room in room_data:
        room_id = room['id']
        room_title = room['title']

        for namespace in room['namespaces']:
            if namespace['id'] == 'devices_all':
                for device in namespace['params']:
                    item_data = get_data(device['deviceId'])['data']

                    if 'deviceType' in item_data:
                        device_type = item_data['deviceType']

                    device_item = {'title': device['deviceName'], 'id': device['deviceId'],
                                   'room': room_title, 'type': device_type}
                    devices.append(device_item)

    if json_data['code'] == 200:
        return {'devices': devices}
    else:
        return "error"


def get_device_value(type, id, db: Database = None):
    if db is None:
        db = Database()


    data = {
        ZWAVE_SENSOR: {'table': 'ZWAVE_SENSOREN', 'id_col': 'ID', 'val_col': 'VALUE'},
        ZWAVE_POWER_METER: {'table': 'ZWAVE_POWER_METER', 'id_col': 'DEVICE_ID', 'val_col': 'VALUE'},
        ZWAVE_THERMOSTAT: {'table': 'ZWAVE_THERMOSTATS', 'id_col': 'THERMOSTAT_ID', 'val_col': 'VALUE'},
        ZWAVE_SWITCH: {'table': 'ZWAVE_SWITCHES', 'id_col': 'ID', 'val_col': 'VALUE'},
        ZWAVE_DIMMER: {'table': 'ZWAVE_DIMMER', 'id_col': 'ID', 'val_col': 'VALUE'}
    }

    if type in data:
        item = db.select_one('SELECT * FROM ' + data[type]['table'] + ' WHERE ' + data[type]['id_col'] + ' = :id',
                             {'id': id})
        return item[data[type]['val_col']]
    else:
        return None


def set_device_value(type, id, value, db: Database = None):
    if db is None:
        db = Database()


    data = {
        ZWAVE_SENSOR: {'table': 'ZWAVE_SENSOREN', 'id_col': 'ID', 'val_col': 'VALUE'},
        ZWAVE_POWER_METER: {'table': 'ZWAVE_POWER_METER', 'id_col': 'DEVICE_ID', 'val_col': 'VALUE'},
        ZWAVE_THERMOSTAT: {'table': 'ZWAVE_THERMOSTATS', 'id_col': 'THERMOSTAT_ID', 'val_col': 'VALUE'},
        ZWAVE_SWITCH: {'table': 'ZWAVE_SWITCHES', 'id_col': 'ID', 'val_col': 'VALUE'},
        ZWAVE_DIMMER: {'table': 'ZWAVE_DIMMER', 'id_col': 'ID', 'val_col': 'VALUE'}
    }

    if type in data:

        # print "Saving: "+type+" - "+id+" - "+str(value)

        if type == ZWAVE_POWER_METER:
            item = db.select_one("SELECT * FROM ZWAVE_POWER_METER WHERE DEVICE_ID = :id", {'id': id})

            try:
                prev_value = float(item['PREV_VALUE'])
                if item['IS_RESET_DAILY'] == 0 and value < prev_value:
                    db.update("UPDATE ZWAVE_POWER_METER SET PREV_VALUE = 0 WHERE DEVICE_ID = :id",
                              {'id': id})
                else:
                    value = value - prev_value
            except:
                value = value

        db.update('UPDATE ' + data[type]['table'] + ' SET ' + data[type]['val_col'] + ' = :value WHERE ' + data[type][
            'id_col'] + ' = :id',
                  {'value': value, 'id': id})

        return True
    else:
        return False



#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Exception import NoSuchTypeException
from Homevee.Item.Device import Device
from Homevee.Item.Device.Switch.Funksteckdose import Funksteckdose
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *


def get_mode(type, id, db: Database = None):
    if db is None:
        db = Database()
    module_map = {
        FUNKSTECKDOSE: Funksteckdose
    }

    if type not in module_map:
        raise NoSuchTypeException("type "+type+" does not exist")

    item = Device.load_from_db(module_map[type], id)

    return item.mode

def get_modes(user, room, type, id, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(room):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    if id is None or id == "":
        modi = []

        devices = {}

        modules = [Funksteckdose]

        for module in modules:
            devices = Device.get_all(module, room, db)

            for device in devices:
                item = device.get_dict()
                modi.append(item)

        return {'modi': modi}

    else:
        return get_mode(type, id, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Functions.tv_data import get_tv_plan
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant import Helper
from Homevee.VoiceAssistant.Modules import VoiceModule


class VoiceGetTvTippsModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            [['was', 'welche'], 'sind', ['tv-tipps', 'tvtipps', 'tv-tips', 'tvtips']],
            [['was', 'welche'], 'sind', ['tv', 'fernseh'], ['tipps', 'tips']],
            [['was', 'welche'], 'sind', ['fernseh-tipps', 'fernsehtipps', 'fernseh-tips', 'fernsehtips']],
        ]

    def get_label(self):
        return "gettvtipps"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.get_tv_tipps(username, text, context, db)

    def get_tv_tipps(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        tv_shows = get_tv_plan(username, "tipps", db)

        VERB = ['kommt', 'läuft']

        show_strings = []
        for show in tv_shows:
            TIME = show['time'] + ' Uhr'
            CHANNEL = show['channel']
            NAME = '\''+show['name']+'\''

            string_data = [
                ['um ', TIME, ' ', VERB, ' auf ', CHANNEL,  ' ', NAME],
                ['um ', TIME, ' ', VERB, ' ', NAME, ' auf ', CHANNEL],
                ['auf ', CHANNEL, ' um ', TIME, ' ', VERB, ' ', NAME],
                ['auf ', CHANNEL, ' ', VERB, ' ', NAME, ' um ', TIME],
                [NAME, ' ', VERB, ' um ', TIME, ' auf ', CHANNEL],
                [NAME, ' ', VERB, ' auf ', CHANNEL, ' um ', TIME],
            ]

            show_strings.append(Helper.generate_string(string_data))

        show_string = None
        for i in range(0, len(show_strings)):
            if show_string is None:
                show_string = show_strings[i]
            elif i == len(show_strings)-1:
                if i != 0:
                    show_string = show_string + ' und '
                show_string = show_string+show_strings[i]
            else:
                show_string = show_string+', '+show_strings[i]

        answer_data = [
            [['Die', 'Deine'], ' ', ['TV', 'Fernseh', 'Programm'], '-', ['Tipps', 'Vorschläge', 'Empfehlungen'], ' ',
                ['heute', 'für heute'], ' ', ['sind', 'lauten'], ':']
        ]

        output = Helper.generate_string(answer_data) + ' ' + show_string + '.'

        return {'msg_speech': output, 'msg_text': output}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.Functions.condition_actions.actions import run_actions
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database


def get_voice_commands(db):
    rules = []
    results = db.select_all("SELECT * FROM CUSTOM_VOICE_COMMANDS", {})

    for item in results:
        commands = get_command_data(item['ID'], db)
        responses = get_response_data(item['ID'], db)

        rules.append({'id': item['ID'], 'name': item['NAME'], 'action_data': item['ACTION_DATA'],
                      'command_data': commands, 'response_data': responses})

    return {'rules': rules}

def get_command_data(id, db: Database = None):
    if db is None:
        db = Database()
    items = db.select_all("SELECT * FROM CUSTOM_VOICE_COMMAND_SENTENCES WHERE COMMAND_ID = :id", {'id': id})

    data = []

    for item in items:
        data.append(item['COMMAND'])

    return data

def get_response_data(id, db: Database = None):
    if db is None:
        db = Database()
    items = db.select_all("SELECT * FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id", {'id': id})

    data = []

    for item in items:
        data.append(item['RESPONSE'])

    return data

def add_edit_voice_command(username, id, name, command_data, response_data, action_data, db: Database = None):
    if db is None:
        db = Database()
    add_new = (id == None or id == "" or id == "-1")

    if(add_new):
        id = db.insert("INSERT INTO CUSTOM_VOICE_COMMANDS (NAME, ACTION_DATA) VALUES (:name, :actions)",
                    {'name': name, 'actions': action_data})

        command_data = json.loads(command_data)
        add_command_data(command_data, id, db)

        response_data = json.loads(response_data)
        add_response_data(response_data, id, db)
        return Status(type=STATUS_OK).get_dict()

    else:
        db.update("UPDATE AUTOMATION_DATA SET NAME = :name, ACTION_DATA = :actions WHERE ID = :id",
            {'name': name, 'actions': action_data, 'id': id})

        db.delete("DELETE FROM CUSTOM_VOICE_COMMAND_SENTENCES WHERE COMMAND_ID = :id", {'id': id})
        db.delete("DELETE FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id", {'id': id})

        command_data = json.loads(command_data)
        add_command_data(command_data, id, db)

        response_data = json.loads(response_data)
        add_response_data(response_data, id, db)

        return Status(type=STATUS_OK).get_dict()

def delete_voice_command(username, id, db: Database = None):
    if db is None:
        db = Database()
    db.delete("DELETE FROM CUSTOM_VOICE_COMMANDS WHERE ID = :id", {'id': id})
    db.delete("DELETE FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id", {'id': id})
    db.delete("DELETE FROM CUSTOM_VOICE_COMMAND_SENTENCES WHERE COMMAND_ID = :id", {'id': id})

    return Status(type=STATUS_OK).get_dict()

def add_command_data(commands, id, db: Database = None):
    if db is None:
        db = Database()
    for command in commands:
        param_array = {'id': id, 'command': command.lower()}

        db.insert("""INSERT INTO CUSTOM_VOICE_COMMAND_SENTENCES (COMMAND_ID, COMMAND) VALUES (:id, :command)""",
            param_array)

def add_response_data(responses, id, db: Database = None):
    if db is None:
        db = Database()
    for response in responses:
        param_array = {'id': id, 'response': response}

        db.insert("""INSERT INTO CUSTOM_VOICE_COMMAND_RESPONSES (COMMAND_ID, RESPONSE) VALUES (:id, :response)""",
            param_array)

def run_custom_voice_commands(text, username, db: Database = None):
    if db is None:
        db = Database()
    result = db.select_one("SELECT * FROM CUSTOM_VOICE_COMMAND_SENTENCES, CUSTOM_VOICE_COMMANDS WHERE ID = COMMAND_ID AND COMMAND = :command",
                {'command': text})

    if(result is None):
        return None

    id = result['ID']

    action_data = result['ACTION_DATA']

    action_data = json.loads(action_data)

    #run actions
    run_actions(action_data, db)

    result = db.select_one("SELECT * FROM CUSTOM_VOICE_COMMAND_RESPONSES WHERE COMMAND_ID = :id ORDER BY RANDOM()",
                {'id': id})

    return result['RESPONSE']


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from Homevee.Functions.calendar import get_calendar_day_items
from Homevee.VoiceAssistant.Helper import *
from Homevee.VoiceAssistant.Modules.CalendarModule import VoiceCalendarModule


class VoiceGetCalendarModule(VoiceCalendarModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            ['was', 'steht', 'an'],
            ['was', 'habe', 'ich', 'vor'],
            ['habe', 'ich', 'termine']
        ]

    def get_label(self):
        return "getcalendar"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.get_calendar(username, text, context, db)

    def get_calendar(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        # find date

        date_word, date = self.find_date(text)

        # date = "2018-05-23"

        calendar_entries = get_calendar_day_items(username, date, db)['calendar_entries']

        entries = []

        for entry in calendar_entries:
            start = datetime.datetime.strptime(entry['start'], '%H:%M')
            dnow = datetime.datetime.now()

            if (dnow.time() > start.time()):
                continue

            entries.append(entry)

        if len(entries) > 0:
            calendar_string = None

            for i in range(0, len(entries)):
                if calendar_string is None:
                    calendar_string = entry['name'] + ' um ' + entry['start'] + ' Uhr'
                elif i == len(entries):
                    calendar_string = ' und ', entry['name'] + ' um ' + entry['start'] + ' Uhr'
                else:
                    calendar_string += ', ' + entry['name'] + ' um ' + entry['start'] + ' Uhr'

            if len(entries) > 1:
                answer_data = [
                    ['Du hast ', date_word, ' ', len(entries), ' ',
                     ['Termine', 'Kalendereinträge', 'Einträge im Kalender'], ': ',
                     calendar_string]
                ]
            else:
                answer_data = [
                    ['Du hast ', date_word, ' einen', ' ', ['Termin', 'Kalendereintrag', 'Eintrag im Kalender'], ': ',
                     calendar_string]
                ]
        else:
            answer_data = [
                ['Du hast ', date_word, ' keine ', ' ', ['Termine', 'Kalendereinträge', 'Einträge im Kalender'], '.']
            ]

        output = generate_string(answer_data)

        return {'msg_speech': output, 'msg_text': output}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import os
import socket
import sys
import time
import traceback
from _thread import start_new_thread

from Homevee import Updater, RestAPI
from Homevee.API import API
from Homevee.CloudConnection import CloudConnection
from Homevee.Cronjob.CalendarReminderCronjob import CalendarReminderCronjob
from Homevee.Cronjob.CheckUpdateCronjob import CheckUpdateCronjob
from Homevee.Cronjob.HeatingSchemeCronjob import HeatingSchemeCronjob
from Homevee.Cronjob.OverviewNotificationCronjob import OverviewNotificationCronjob
from Homevee.Cronjob.PhilipsHueValueLoaderCronjob import PhilipsHueValueLoaderCronjob
from Homevee.Cronjob.SaveEnergyDataCronjob import SaveEnergyDataCronjob
from Homevee.Cronjob.SaveSensorDataCronjob import SaveSensorDataCronjob
from Homevee.Cronjob.TimedAutomationsCronjob import TimedAutomationsCronjob
from Homevee.Cronjob.UserPingCronjob import UserPingCronjob
from Homevee.Cronjob.WeatherUpdaterCronjob import WeatherUpdaterCronjob
from Homevee.Cronjob.ZWaveValueLoaderCronjob import ZWaveValueLoaderCronjob
from Homevee.Helper import Logger, translations
from Homevee.Helper.translations import translate, translate_print
from Homevee.Item.User import User
from Homevee.SmartSpeakerConnection import SmartSpeakerConnection
from Homevee.Utils.Constants import END_OF_MESSAGE
from Homevee.Utils.Database_NEW import Database
from . import Cronjob
from . import VoiceAssistant
from .DeviceAPI import mqtt_api
# from Functions import ar_control, people_classifier
# from Helper import compression
from .Helper.helper_functions import save_request_to_db, send_to_client, \
    update_ip_thread, update_cert_thread, check_cert


class Homevee():
    #add flags
    def __init__(self, cloud_connection=True, websocket_server=True, http_server=True):
        self.cloud_connection = cloud_connection
        self.websocket_server = websocket_server
        self.http_server = http_server

        db = Database()
        db.upgrade()

        translate_print('homevee_server_started')

        homevee_update = Updater.get_homevee_update_version()

        if(homevee_update is not None):
            print(translate('update_available').format(homevee_update))

        self.init_cronjobs()

    def add_user(self, username, password, is_admin=False, db: Database = None):
        if db is None:
            db = Database()

        hashed_pw, salt = User.hash_password(password)

        permissions = ''
        if is_admin:
            permissions = '{"permissions":["admin"]}'

        param_array = {'username': username, 'ip': '', 'permissions': permissions, 'password': hashed_pw, 'salt': salt}
        db.insert("INSERT OR IGNORE INTO 'userdata' (USERNAME, PASSWORD, PW_SALT, IP, PERMISSIONS) VALUES (:username, :password, :salt, :ip, :permissions);",
            param_array)

    def init_cronjobs(self):
        CalendarReminderCronjob()
        HeatingSchemeCronjob()
        TimedAutomationsCronjob()

        SaveEnergyDataCronjob()
        SaveSensorDataCronjob()
        OverviewNotificationCronjob()
        PhilipsHueValueLoaderCronjob()
        ZWaveValueLoaderCronjob()

        WeatherUpdaterCronjob()
        CheckUpdateCronjob()

        #UserPingCronjob()

    def start(self):
        HOST = ''  # Symbolic name meaning all available interfaces
        PORT = 7777  # Arbitrary non-privileged port

        # Zeitzone anpassen
        os.environ['TZ'] = 'Europe/Berlin'

        '''while True:
            data = raw_input("Data: ")
            key = raw_input("Key: ")

            aes = AESCipher.AESCipher(key)
            new_aes = AESCipher.AESCipher(key)
            Logger.log("Data: "+data)
            cipher = aes.encrypt(data)
            Logger.log("Cipher: "+cipher)
            plain = new_aes.decrypt(cipher)
            Logger.log("Plain: "+plain)'''

        # aes = AESCipher.AESCipher('3s6v9y$B&E)H@McQfTjWmZq4t7w!z%C*')
        # Logger.log("Plain: "+aes.decrypt('qYqXviw/9yowSeMeJqkRhC6QUtQdPjFhyUdlzb0Cyz4xddOQbLMpNUjs8hMbk8al'))

        # Cronjob.save_energy_data.init_thread()

        test_speech = False
        if test_speech:
            while (True):
                voice_input = input("Gib einen Sprachbefehl ein: ")
                print(VoiceAssistant.voice_command("sascha", voice_input, None, None, Database(), translations.LANGUAGE))

        # Smart Speaker Communication
        #smart_speaker_connection = SmartSpeakerConnection()
        #smart_speaker_connection.start_loop()

        # Connect to MQTT-Broker
        start_new_thread(mqtt_api.init_client, ())

        # Start Websocket-Server
        if self.websocket_server:
            pass
            # start_new_thread(start_websocket_server, ())

        # Start HTTP-Server
        if self.http_server:
            pass
            # start_new_thread(webinterface.start_http_server, ())

        # Communicate with cloud
        if self.cloud_connection:
            #start_new_thread(cloud_connection_loop, ())

            #cloud_connection = CloudConnection()
            #cloud_connection.start_connection_loop()

            start_new_thread(CloudConnection.cloud_connection_loop, ())

        # Lokale IP und Zertifikat regelmäßig aktualisieren
        start_new_thread(update_ip_thread, ())
        start_new_thread(update_cert_thread, ())

        # Reset Image-Classifier flags
        # db = Database()
        # db.set_server_data(ar_control.IS_TRAINING_TAG, "false", db)
        # db.set_server_data(people_classifier.IS_TRAINING_TAG, "false", db)

        #RestAPI.start_rest_api()

        # check if certs exist
        check_cert(db=Database())

        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        Logger.log('Socket created')

        # Bind socket to local host and port
        try:
            s.bind((HOST, PORT))
        except socket.error as msg:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            Logger.log('Bind failed. Error Code : ' + str(msg[0]) + ' Message ' + msg[1])
            sys.exit()

        Logger.log('Socket bind complete')

        # Start listening on socket
        s.listen(10)
        Logger.log('Socket now listening')

        while True:
            try:
                self.listen_for_requests(s)
            except KeyboardInterrupt:
                break
            except:
                if(Logger.IS_DEBUG):
                    traceback.print_exc()

        s.close()

    # Function for handling connections. This will be used to create threads
    def clientthread(self, conn):
        # infinite loop so that function do not terminate and thread do not end.
        db = Database()

        is_http = False

        sent_data = None

        while True:
            try:
                # Receiving from client
                method_start_time = time.time()

                data = ""

                while (True):
                    new_data = conn.recv(8192).decode('utf-8')

                    # Logger.log(new_data)

                    # data = data + compression.decompress_string(new_data)

                    data = data + new_data

                    if (data.endswith(END_OF_MESSAGE)):
                        break

                if data == "":
                    db.close()
                    conn.close()
                    return

                data = data[:len(END_OF_MESSAGE) * -1]

                Logger.log("Received: " + str(data))

                is_http = False
                is_resend = False

                error = False

                try:
                    data = json.loads(data)
                    data = json.loads(data['msg'])
                    Logger.log("Request is not in HTTP-Format")
                except:
                    Logger.log("Data could not be parsed")
                    send_to_client(json.dumps({'status': 'error'}), conn, is_http)
                    error = True

                if 'resend' in data and data['resend'] is True:
                    # Daten erneut senden
                    Logger.log("Resend!")
                    is_resend = True
                    Logger.log(str(len(sent_data)) + " | " + sent_data)

                    send_to_client(sent_data[data['resendstart']:], conn, is_http)

                if not error and not is_resend:
                    if not is_resend:
                        message = API().process_data(data, db)

                        # print "compressing: "+message

                        # start_time = time.time()
                        # compressed_message = compression.compress_string(message)
                        # end_time = time.time()

                        # print "uncompressed: "+str(len(message))
                        # print "compressed: "+str(len(compressed_message))+", time: "+str(end_time-start_time)

                        # compressed_message = compressed_message.decode('iso-8859-1').encode('utf8')

                        msg = json.dumps({'msg': message, 'computing_time': (time.time() - method_start_time) * 1000})

                    if (msg is not None):
                        send_to_client(msg, conn, is_http)

                        sent_data = msg

                        save_request_to_db(data, msg, db)
                    else:
                        Logger.log("Error in reply")
                        send_to_client(json.dumps({'status': 'error'}), conn, is_http)

                    if not data:
                        break
            # except socket.error:
            #    break
            except Exception as e:
                if(Logger.IS_DEBUG):
                    traceback.print_exc()
                Logger.log("Fehler: " + str(e))
                send_to_client(json.dumps({'status': 'error'}), conn, is_http)

            # Stop endless loop
            break

        # came out of loop
        conn.close()
        db.commit()
        db.close()
        Logger.log("Verbindung beendet")

    def listen_for_requests(self, s):
        # s = ssl.wrap_socket(s, keyfile=constants.LOCAL_SSL_PRIVKEY, certfile=constants.LOCAL_SSL_CERT, server_side=True)

        # wait to accept a connection - blocking call
        conn, addr = s.accept()
        Logger.log('Connected with ' + addr[0] + ':' + str(addr[1]))

        # start new thread takes 1st argument as a function name to be run, second is the tuple of arguments to the function.
        start_new_thread(self.clientthread, (conn,))


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Utils.Database_NEW import Database


def get_places(user, keyword, lat, lng, db: Database = None):
    if db is None:
        db = Database()
    places = []

    return places

def get_place_data(user, place_id, db: Database = None):
    if db is None:
        db = Database()
    data = {}

    return data


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from Homevee.Item import Item
from Homevee.Item.HomeBudgetItem import HomeBudgetItem
from Homevee.Utils.Database_NEW import Database


def get_home_budget_data(user, startdate, db: Database = None):
    if db is None:
        db = Database()
    data = {}

    #Find expenditures grouped by date
    results = db.select_all("SELECT SUM(AMOUNT) as AMOUNT, strftime(\"%d.%m.%Y\", DATE) as DATE FROM HOME_BUDGET_DATA WHERE AMOUNT <= 0 AND DATE >= :date GROUP BY DATE ORDER BY DATE ASC",
                {'date': startdate})
    costs = []

    last_date = None

    for item in results:
        this_date = datetime.datetime.strptime(item['DATE'], "%d.%m.%Y")
        if(last_date is not None):
            difference = (this_date - last_date).days

            for i in range(1, difference):
                date = last_date + datetime.timedelta(days=i)
                costs.append({'date': date.strftime("%d.%m.%Y"), 'amount': round(0, 2)})

        last_date = this_date

        costs.append({'date': item['DATE'], 'amount': round(item['AMOUNT']*-1, 2)})
    data['costs'] = costs

    #Find amount money left
    item = db.select_one("SELECT SUM(AMOUNT) as AMOUNT FROM HOME_BUDGET_DATA", {})
    overall_amount = round(item['AMOUNT'], 2)
    data['overall_amount'] = overall_amount

    #Compute date where money reaches 0
    item = db.select_one("SELECT AVG(AMOUNT) as AMOUNT FROM (SELECT SUM(AMOUNT) as AMOUNT FROM HOME_BUDGET_DATA WHERE AMOUNT < 0 GROUP BY DATE);", {})
    average_spending_per_day = item['AMOUNT']*-1

    #compute timeperiod
    min_max_date = db.select_one("SELECT MIN(DATE) as mindate, MAX(DATE) as maxdate, COUNT(*) as numdays FROM (SELECT DISTINCT DATE FROM HOME_BUDGET_DATA);", {})
    real_numdays = (datetime.datetime.today()-datetime.datetime.strptime(min_max_date['mindate'], "%Y-%m-%d")).days
    average_spending_per_day = average_spending_per_day * min_max_date['numdays'] / real_numdays
    days_left = overall_amount/average_spending_per_day
    date = datetime.datetime.today() + datetime.timedelta(days=days_left)
    date = datetime.datetime.strftime(date, "%d.%m.%Y")
    data['sufficient_until'] = date

    return {'data': data}

def get_home_budget_data_graph(user, startdate, enddate, db: Database = None):
    if db is None:
        db = Database()
    item = db.select_one("SELECT SUM(AMOUNT) as AMOUNT FROM HOME_BUDGET_DATA WHERE DATE < :startdate;",
        {'startdate': startdate})

    startvalue = item['AMOUNT']

    if startvalue is None:
        startvalue = 0

    #Get values
    results = db.select_all("SELECT SUM(AMOUNT) as AMOUNT, strftime(\"%d.%m.%Y\", DATE) as FORMATTED_DATE FROM HOME_BUDGET_DATA WHERE DATE >= :startdate AND DATE <= :enddate GROUP BY DATE ORDER BY DATE ASC",
        {'startdate': startdate, 'enddate': enddate})

    values = []

    last_date = None

    for item in results:
        this_date = datetime.datetime.strptime(item['FORMATTED_DATE'], "%d.%m.%Y")
        if (last_date is not None):
            difference = (this_date - last_date).days

            for i in range(1, difference):
                date = last_date + datetime.timedelta(days=i)
                values.append({'date': date.strftime("%d.%m.%Y"), 'amount': round(0, 2)})

        last_date = this_date

        values.append({'date': item['FORMATTED_DATE'], 'amount': round(item['AMOUNT'], 2)})

    return {'startvalue': startvalue, 'values': values}


def get_home_budget_data_day_items(user, date, db: Database = None):
    if db is None:
        db = Database()
    data = []

    results = db.select_all("SELECT *, strftime(\"%d.%m.%Y\", DATE) as FORMATTED_DATE FROM HOME_BUDGET_DATA WHERE DATE = :date ORDER BY DATE ASC",
                {'date': date})

    for item in results:
        data.append({'id': item['ID'], 'date': item['FORMATTED_DATE'], 'amount': round(item['AMOUNT'], 2), 'info': item['INFO']})

    return {'data': data}

def add_edit_home_budget_data(user, id, date, info, amount, db: Database = None):
    if db is None:
        db = Database()
    home_budget_item = None
    if id is not None:
        home_budget_item = Item.load_from_db(HomeBudgetItem, id)
    if home_budget_item is None:
        home_budget_item = HomeBudgetItem(date, info, amount, id)
    else:
        home_budget_item.date = date
        home_budget_item.info = info
        home_budget_item.amount = amount

    return home_budget_item.api_save()

def delete_home_budget_data(user, id, db: Database = None):
    if db is None:
        db = Database()
    item = Item.load_from_db(HomeBudgetItem, id)
    return item.api_delete()


import traceback

from pymax.cube import Cube

from Homevee.DeviceAPI import max_cube_control
from Homevee.Exception import DatabaseSaveFailedException
from Homevee.Helper import Logger
from Homevee.Item.Device.Thermostat import Thermostat
from Homevee.Item.Gateway import *
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import MAX_THERMOSTAT


class MaxThermostat(Thermostat):
    def __init__(self, name, icon, location, id=None, temp=None):
        super(MaxThermostat, self).__init__(name, icon, location, id=id, temp=temp)

    def get_device_type(self):
        return MAX_THERMOSTAT

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.delete("DELETE FROM MAX_THERMOSTATS WHERE ID == :id", {'id': self.id})
            return True
        except:
            return False

    def get_min_max(self):
        return 0, 30

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            # insert
            db.insert("""INSERT OR IGNORE INTO MAX_THERMOSTATS (ID, NAME, ICON, LAST_TEMP, ROOM) VALUES 
                        (:id, :name, :icon, :last_temp, :room)""",
                        {'id': self.id, 'name': self.name, 'icon': self.icon,
                        'last_temp': self.temp, 'room': self.location})
            # update
            db.update("""UPDATE OR IGNORE MAX_THERMOSTATS SET NAME = :name, ICON = :icon,
                        LAST_TEMP = :last_temp, ROOM = :room WHERE ID = :id""",
                        {'name': self.name, 'icon': self.icon, 'last_temp': self.temp,
                        'room': self.location, 'id': self.id})
                #TODO add generated id to object
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save max-thermostat to database")

    def build_dict(self):
        dict = {
            'name': self.name,
            'icon': self.icon,
            'id': self.id,
            'temp': self.temp,
            'location': self.location
        }
        return dict

    def update_temp(self, temp, db: Database = None):
        if db is None:
            db = Database()
        try:
            gateway = Item.load_from_db(Gateway, MAX_CUBE)

            data = max_cube_control.get_device_data(gateway.ip, self.id)

            # temperatur berechnen
            tempVal = temp

            with Cube(gateway.ip) as cube:
                result = cube.set_mode_manual(data['room'], data['addr'], tempVal)

            return True

        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            return False

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return MaxThermostat.load_all_from_db('SELECT * FROM MAX_THERMOSTATS WHERE ID IN (%s)'
                                              % ','.join('?' * len(ids)), ids, db)

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = MaxThermostat(result['NAME'], result['ICON'], result['RAUM'], result['ID'],
                                 result['LAST_TEMP'])
            items.append(item)
        return items

    @staticmethod
    def load_all(db: Database):
        return MaxThermostat.load_all_from_db('SELECT * FROM MAX_THERMOSTATS', {}, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            name = dict['name']
            id = dict['id']
            location = dict['location']
            temp = dict['temp']
            icon = dict['icon']

            item = MaxThermostat(name, icon, location, id, temp)

            return item

        except:
            raise InvalidParametersException("Invalid parameters for MaxThermostat.create_from_dict()")


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant.Modules import VoiceModule

class VoicePlacesApiModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            [['wo', 'was'], 'ist', ['der', 'die', 'das'], ['nähste', 'nächste']]
        ]

    def get_label(self):
        return "placesapi"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.get_places(username, text, context, db)

    def get_places(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return {'msg_speech': 'Places', 'msg_text': 'Places'}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database


def get_voice_replace_items(user, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    replace = []

    results = db.select_all("SELECT DISTINCT REPLACE_WITH FROM VOICE_COMMAND_REPLACE WHERE USERNAME = :user",
                {'user': user.username})

    for item in results:
        items = db.select_all("SELECT TEXT FROM VOICE_COMMAND_REPLACE WHERE REPLACE_WITH == :item AND USERNAME = :user",
            {'user': user.username, 'item': item['REPLACE_WITH']})

        replacements = []

        for replacement in items:
            replacements.append(replacement['TEXT'])

        replace.append({'replacewith': item['REPLACE_WITH'], 'replacearray': replacements})

    return {'replacedata': replace}

def add_edit_voice_replace_item(user, replacewith, replaceitems, db: Database = None):
    if db is None:
        db = Database()
    replaceitems = json.loads(replaceitems)

    db.delete("DELETE FROM VOICE_COMMAND_REPLACE WHERE REPLACE_WITH = :replacewith AND USERNAME = :user",
                {'replacewith': replacewith, 'user': user.username})

    for item in replaceitems:
        db.insert("INSERT INTO VOICE_COMMAND_REPLACE (user, REPLACE_WITH, TEXT) VALUES (:user, :replacewith, :text)",
                    {'user': user.username, 'replacewith': replacewith, 'text': item})

    return Status(type=STATUS_OK).get_dict()

def delete_voice_replace_item(user, replacewith, db: Database = None):
    if db is None:
        db = Database()
    db.delete("DELETE FROM VOICE_COMMAND_REPLACE WHERE REPLACE_WITH = :replacewith AND USERNAME = :user",
                {'replacewith': replacewith, 'user': user.username})
    return Status(type=STATUS_OK).get_dict()


IS_DEBUG = True

def log(log_data):
    """
    Prints the given string to command line if IS_DEBUG is true
    :param log_data: the data to log
    :return:
    """
    if IS_DEBUG:
        print(log_data)


#!/usr/bin/python
# # -*- coding: utf-8 -*-
import datetime

from Homevee.Cronjob import FixedTimeCronjob
from Homevee.Helper import Logger
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *


class SaveEnergyDataCronjob(FixedTimeCronjob):
    def __init__(self):
        super(FixedTimeCronjob, self).__init__(task_name="SaveEnergyDataCronjob")

    def task_to_do(self, *args):
        self.save_energy_data()

    def get_seconds_to_wait(self, execution_time=None):
        t = datetime.datetime.today()

        seconds_to_wait = (24 * 60 * 60) - (t.hour * 60 * 60) - (t.minute * 60) - t.second - 60  # um 23:59 ausführen

        return seconds_to_wait

    def save_energy_data(self):

        db = Database()

        with db:
            date_now = datetime.datetime.now()
            date = date_now.strftime('%Y-%m-%d %H:%M')

            total_value = 0

            room_total_value = {}

            # Z-Wave Stromzähler
            results = db.select_all("SELECT * FROM ZWAVE_POWER_METER")
            for item in results:
                room_id = item['ROOM_ID']
                value = self.save_to_db(date_now, ZWAVE_POWER_METER, room_id, item['DEVICE_ID'],
                                   item['VALUE'])
                total_value += value

                if room_id in room_total_value:
                    room_total_value[room_id] += value
                else:
                    room_total_value[room_id] = value

            #Gesamtwerte der Räume speichern
            for room in room_total_value:
                Logger.log("inserting in room_energy_data of room: "+str(room))
                db.insert("INSERT INTO ROOM_ENERGY_DATA (ROOM_ID, POWER_USAGE, DATE) VALUES (:room, :usage, :date)",
                            {'room': room, 'usage': room_total_value[room_id], 'date': date})

            #Gesamtverbrauch speichern
            Logger.log("inserting in energy_data")
            db.insert("INSERT INTO ENERGY_DATA (DATE, POWER_USAGE) VALUES (:date, :value)",
                        {'date': date, 'value': total_value})

    def save_to_db(self, date_now, type, room_id, id, value, db: Database = None):
        if db is None:
            db = Database()
        if value is None or value == "N/A":
            return

        value = float(value)

        with db:
            date = date_now.strftime('%Y-%m-%d %H:%M')

            if type == ZWAVE_POWER_METER:
                result = db.select_one("SELECT PREV_VALUE FROM ZWAVE_POWER_METER WHERE DEVICE_ID = :id", {'id': id})
                prev_value = float(result['PREV_VALUE'])

                prev_value = prev_value+value
                Logger.log("prev_value: "+str(prev_value))
                Logger.log("updating zwave_power_meter value")
                db.update("UPDATE ZWAVE_POWER_METER SET PREV_VALUE = :val, VALUE = 0 WHERE DEVICE_ID = :id",
                    {'val': prev_value, 'id': id})

            Logger.log("inserting in device_energy_data")
            db.insert("INSERT INTO DEVICE_ENERGY_DATA (LOCATION, DEVICE_ID, DEVICE_TYPE, DATE, POWER_USAGE) \
                            VALUES (:location, :id, :type, :date, :value)",
                        {'location': room_id, 'id': id, 'type': type, 'date': date, 'value': value})

            return value



#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.DeviceAPI.get_modes import get_modes
from Homevee.DeviceAPI.set_modes import set_modes
from Homevee.DeviceAPI.wake_on_lan import wake_on_lan
from Homevee.Helper import Logger
from Homevee.Item.Status import *
from Homevee.Manager.dashboard import get_device_info
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *


def get_rfid_tags(username, db: Database = None):
    if db is None:
        db = Database()

    results = db.select_all("SELECT * FROM RFID_TAGS")

    tags = []

    for item in results:
        try:
            device_info = get_device_info(username, item['ACTION_TYPE'], item['ACTION_ID'], db)

            Logger.log(item)
            Logger.log(device_info)

            tag = {'name': item['NAME'], 'uuid': item['UUID'], 'actiontype': item['ACTION_TYPE'],
                   'actionid': item['ACTION_ID'], 'roomname': device_info['roomname'], 'actionname': device_info['name']}
            tags.append(tag)
        except:
            continue

    return tags

def add_edit_rfid_tag(username, name, uuid, actionType, actionId, db: Database = None):
    if db is None:
        db = Database()

    result = db.select_one("SELECT COUNT(*) FROM RFID_TAGS WHERE UUID = :uuid", {'uuid': uuid})

    is_new = result['COUNT(*)'] is not 1

    param_array = {'name': name, 'uuid': uuid, 'type': actionType, 'id': actionId}

    if is_new:
        db.insert("INSERT INTO RFID_TAGS (NAME, UUID, ACTION_TYPE, ACTION_ID) VALUES (:name, :uuid, :type, :id)",
                    param_array)
    else:
        db.update("UPDATE RFID_TAGS SET NAME = :name, ACTION_TYPE = :type, ACTION_ID = :id WHERE UUID = :uuid",
                    param_array)

    return Status(type=STATUS_OK).get_dict()

def delete_rfid_tag(userane, uuid, db: Database = None):
    if db is None:
        db = Database()

    db.delete("DELETE FROM RFID_TAGS WHERE UUID = :uuid", {'uuid': uuid})

    return Status(type=STATUS_OK).get_dict()


def wake_xbob_on_lan(username, id, db: Database = None):
    if db is None:
        db = Database()
    pass


def run_rfid_action(username, uuid, db: Database = None):
    if db is None:
        db = Database()
    result = db.select_one("SELECT * FROM RFID_TAGS WHERE UUID = :uuid", {'uuid': uuid})

    if result is None:
        return {'result': 'tagnotfound'}
    else:
        type = result['ACTION_TYPE']
        id = result['ACTION_ID']

        if type in [FUNKSTECKDOSE, ZWAVE_SWITCH, URL_SWITCH, URL_TOGGLE]:
            mode = get_modes(username, None, type, id, db)

            if mode == "1" or mode == 1 or mode == True or mode == "true":
                mode = 0
            else:
                mode = 1

            result = set_modes(username, type, id, mode, db)

        elif type == WAKE_ON_LAN:
            result = wake_on_lan(username, id, db)

        elif type == XBOX_ONE_WOL:
            result = wake_xbob_on_lan(username, id, db)

    return Status(type=STATUS_OK).get_dict()


from Homevee.Exception import AbstractFunctionCallException
from Homevee.Item.Device import Device
from Homevee.Utils.Database_NEW import Database


class Switch(Device):
    def __init__(self, name, icon, location, id=None, mode=False):
        super(Switch, self).__init__(name, icon, location, id=id)

        if mode == 1:
            self.mode = True
        else:
            self.mode = False

    def set_mode(self, mode: bool, db: Database = None):
        if db is None:
            db = Database()
        """
        Sets the mode of the switch-device
        :param mode: the new mode
        :param db: the database connection
        :return:
        """
        #TODO sicherstellen, dass mode immer boolean ist (Ticket #26)
        if mode == "1" or mode == 1 or mode == True:
            mode = True
        else:
            mode = False

        if(self.update_mode(mode)):
            self.mode = mode
            self.save_to_db(db)

    def update_mode(self, mode: bool, db: Database = None) -> bool:
        """
        Update the mode of the switch-device
        :param mode: the mode to set the device to
        :param db: the database connection
        :return: true if the mode has been successfully updated, false otherwise
        """
        raise AbstractFunctionCallException("Switch.update_mode() is abstract")

    #@staticmethod
    #def get_all(location=None, db: Database = None):
    #    if db is None:
    #        db = Database()
    #    # get all devices of all types
    #    devices = []
    #    devices.extend(Switch.get_all(location))
    #    return devices


from Homevee.Item import Item
from Homevee.Item.Logic.Action import Action
from Homevee.Item.Scene import Scene


class RunSceneAction(Action):
    def __init__(self, id):
        super(RunSceneAction, self).__init__()
        self.id = id

        self.scene = Item.load_from_db(Scene, self.id)

    def run(self):
        self.scene.run()

    @staticmethod
    def get_from_dict(dict):
        try:
            id = dict['id']
            return RunSceneAction(id)
        except:
            return None


#!/usr/bin/python
# -*- coding: utf-8 -*-

from Homevee.Cronjob import IntervalCronjob
from Homevee.Utils.Database_NEW import Database


class CalendarReminderCronjob(IntervalCronjob):
    def __init__(self):
        super(CalendarReminderCronjob, self).__init__(task_name="CalendarReminder", interval_seconds=60)

    def task_to_do(self, *args):
        self.remind_users()

    def remind_users(self):
        db = Database()
        return


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib.error
import urllib.parse
import urllib.request


def get_api_key(cube_ip):
    contents = urllib.request.urlopen("http://"+cube_ip+"/api/link").read()

    data = json.loads(contents)

    if(data['status'] == 'success'):
        api_key = data['apiKey']

        #save_api_key

        return True
    elif(data['status'] == 'error'):
        #maybe forgot to press button?
        return False


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import re

from Homevee.VoiceAssistant.Helper import *
from Homevee.VoiceAssistant.Modules import VoiceModule


class VoiceCalendarModule(VoiceModule):
    def find_date(self, text):
        now = datetime.datetime.now()

        try:
            month_replacements = [
                ['. januar ', '.01.'],
                ['. februar ', '.02.'],
                ['. märz ', '.03.'],
                ['. april ', '.04.'],
                ['. mai ', '.05.'],
                ['. juni ', '.06.'],
                ['. juli ', '.07.'],
                ['. august ', '.08.'],
                ['. september ', '.09.'],
                ['. oktober ', '.10.'],
                ['. november ', '.11.'],
                ['. dezember ', '.12.'],

                ['.januar ', '.01.'],
                ['.februar ', '.02.'],
                ['.märz ', '.03.'],
                ['.april ', '.04.'],
                ['.mai ', '.05.'],
                ['.juni ', '.06.'],
                ['.juli ', '.07.'],
                ['.august ', '.08.'],
                ['.september ', '.09.'],
                ['.oktober ', '.10.'],
                ['.november ', '.11.'],
                ['.dezember ', '.12.'],
            ]

            for replacement in month_replacements:
                text = text.replace(replacement[0], replacement[1])

            date = re.search('\d{2}\.\d{2}\.\d{4}', text)
            date_formatted = datetime.datetime.strptime(date.group(), '%d.%m.%Y').date()

            return 'am ' + date_formatted.strftime("%d.%m.%Y"), date_formatted.strftime("%Y-%m-%d")
        except Exception as e:
            Logger.log("No date found")

            try:
                month_replacements = [
                    ['. januar ', '.01. '],
                    ['. februar ', '.02. '],
                    ['. märz ', '.03. '],
                    ['. april ', '.04. '],
                    ['. mai ', '.05. '],
                    ['. juni ', '.06. '],
                    ['. juli ', '.07. '],
                    ['. august ', '.08. '],
                    ['. september ', '.09. '],
                    ['. oktober ', '.10. '],
                    ['. november ', '.11. '],
                    ['. dezember ', '.12. '],

                    ['.januar ', '.01. '],
                    ['.februar ', '.02. '],
                    ['.märz ', '.03. '],
                    ['.april ', '.04. '],
                    ['.mai ', '.05. '],
                    ['.juni ', '.06. '],
                    ['.juli ', '.07. '],
                    ['.august ', '.08. '],
                    ['.september ', '.09. '],
                    ['.oktober ', '.10. '],
                    ['.november ', '.11. '],
                    ['.dezember ', '.12. '],
                ]

                for replacement in month_replacements:
                    text = text.replace(replacement[0], replacement[1])

                date = re.search('\d{2}\.\d{2}\.', text)
                date_formatted = datetime.datetime.strptime(date.group()+str(datetime.datetime.now().year), '%d.%m.%Y').date()

                return 'am ' + date_formatted.strftime("%d.%m.%Y"), date_formatted.strftime("%Y-%m-%d")
            except Exception as e:
                Logger.log("Date without year also not found")

        words = text.split()

        if "heute" in words:
            return 'heute', now.strftime("%Y-%m-%d")
        if "morgen" in words:
            return 'morgen', (now + datetime.timedelta(days=1)).strftime("%Y-%m-%d")
        if "übermorgen" in words:
            return 'übermorgen', (now + datetime.timedelta(days=2)).strftime("%Y-%m-%d")

        #when no date was found return today
        return 'heute', now.strftime("%Y-%m-%d")


import json
import traceback
import urllib.request

from Homevee.Exception import DatabaseSaveFailedException
from Homevee.Helper import Logger
from Homevee.Item.Device.Thermostat import Thermostat
from Homevee.Item.Gateway import *
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import RADEMACHER_THERMOSTAT


class RademacherThermostat(Thermostat):
    def __init__(self, name, icon, location, id=None, temp=None):
        super(RademacherThermostat, self).__init__(name, icon, location, id=id, temp=temp)

    def get_device_type(self):
        return RADEMACHER_THERMOSTAT

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.delete("DELETE FROM HOMEPILOT_THERMOSTATS WHERE ID == :id", {'id': self.id})
            return True
        except:
            return False

    def get_min_max(self):
        return 4, 28

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.insert("""INSERT OR IGNORE INTO HOMEPILOT_THERMOSTATS (ID, NAME, ICON, LAST_TEMP, ROOM) VALUES 
                                        (:id, :name, :icon, :last_temp, :room)""",
                              {'id': self.id, 'name': self.name, 'icon': self.icon,
                               'last_temp': self.temp, 'room': self.location})
            # update
            db.update("""UPDATE OR IGNORE HOMEPILOT_THERMOSTATS SET NAME = :name, ICON = :icon,
                                        LAST_TEMP = :last_temp, ROOM = :room WHERE ID = :id""",
                              {'name': self.name, 'icon': self.icon, 'last_temp': self.temp,
                               'room': self.location, 'id': self.id})

                # TODO add generated id to object
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save homepilot-thermostat to database")

    def build_dict(self):
        dict = {
            'name': self.name,
            'icon': self.icon,
            'id': self.id,
            'temp': self.temp,
            'location': self.location
        }
        return dict

    def update_temp(self, temp, db: Database = None):
        if db is None:
            db = Database()
        try:
            gateway = Item.load_from_db(Gateway, RADEMACHER_HOMEPILOT)

            value = int(float(temp) * 10)

            url = "http://" + gateway.ip + "/deviceajax.do?cid=9&did=" + str(self.id) + "&goto=" + str(value) + "&command=0"

            response = urllib.request.urlopen(url).read()

            data = json.loads(response)

            if (data['status'] != 'uisuccess'):
                return False
            return True
        except:
            if Logger.IS_DEBUG:
                traceback.print_exc()
        return False

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return RademacherThermostat.load_all_from_db('SELECT * FROM HOMEPILOT_THERMOSTATS WHERE ID IN (%s)'
                                              % ','.join('?' * len(ids)), ids)

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = RademacherThermostat(result['NAME'], result['ICON'], result['ROOM'], result['ID'],
                                 result['LAST_TEMP'])
            items.append(item)
        return items

    @staticmethod
    def load_all(db: Database):
        return RademacherThermostat.load_all_from_db('SELECT * FROM HOMEPILOT_THERMOSTATS', {})

    @staticmethod
    def create_from_dict(dict):
        try:
            name = dict['name']
            id = dict['id']
            location = dict['location']
            temp = dict['temp']
            icon = dict['icon']

            item = RademacherThermostat(name, icon, location, id, temp)

            return item

        except:
            raise InvalidParametersException("Invalid parameters for RademacherThermostat.create_from_dict()")


from Homevee.Exception import InvalidParametersException, AbstractFunctionCallException
from Homevee.Item import Item
from Homevee.Utils.Database_NEW import Database

PHILIPS_HUE_BRIDGE = "Philips Hue"
Z_WAVE_GATEWAY = "Z-Wave"
FUNKSTECKDOSEN_CONTROLLER = "Funksteckdosen-Controller"
MAX_CUBE = "MAX! Cube"
MQTT_BROKER = "MQTT Broker"
MIYO_CUBE = "MIYO Cube"
RADEMACHER_HOMEPILOT = "Rademacher HomePilot"


class Gateway(Item):
    def __init__(self, id, ip, port, key1, key2, type):
        super(Gateway, self).__init__()
        self.id = id
        self.ip = ip
        self.port = port
        self.key1 = key1
        self.key2 = key2
        self.type = type

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            if db is None:
                db = Database()



                db.insert("""INSERT OR IGNORE INTO GATEWAYS (NAME, IP, PORT, KEY1, KEY2, TYPE) VALUES
                            (:name, :ip, :port, :key1, :key2, :type)""",
                            {'name': self.id, 'ip': self.ip, 'port': self.port,
                             'key1': self.key1, 'key2': self.key2, 'type': self.type})

                db.update("""UPDATE OR IGNORE GATEWAYS SET IP = :ip, PORT = :port, KEY1 = :key1,
                            KEY2 = :key2, TYPE = :type WHERE NAME = :name""",
                                {'ip': self.ip, 'port': self.port, 'key1': self.key1,
                             'key2': self.key2, 'type': self.type, 'name': self.id})

                return True
        except:
            return False

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.delete("DELETE FROM GATEWAYS WHERE NAME == :key", {'key': self.id})
            return True
        except:
            return False

    def build_dict(self):
        dict = {
            'name': self.id,
            'ip': self.ip,
            'port': self.port,
            'key1': self.key1,
            'key2': self.key2,
            'type': self.type
        }
        return dict

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return Gateway.load_all_from_db('SELECT * FROM GATEWAYS WHERE NAME IN (%s)' % ','.join('?' * len(ids)),
                                        ids, db)

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        gateways = []
        for item in db.select_all(query, params):
            gateway = Gateway(item['NAME'], item['IP'], item['PORT'], item['KEY1'],
                              item['KEY2'], item['TYPE'])
            gateways.append(gateway)
        return gateways

    @staticmethod
    def create_from_dict(dict):
        try:
            name = dict['name']
            ip = dict['ip']
            port = dict['port']
            key1 = dict['key1']
            key2 = dict['key2']
            type = dict['type']

            gateway = Gateway(name, ip, port, key1, key2, type)

            return gateway
        except:
            raise InvalidParametersException("Invalid parameters for Gateway.create_from_dict()")

    def get_devices(self):
        raise AbstractFunctionCallException("Method Gateway.get_devices() is abstract")



STATUS_OK = 'ok'
STATUS_ERROR = 'error'
STATUS_NO_ADMIN = 'noadmin'
STATUS_NO_PERMISSION = 'nopermission'
STATUS_NO_SUCH_TYPE = 'nosuchtype'
STATUS_USER_NOT_FOUND = 'usernotfound'
STATUS_WRONG_DATA = 'wrongdata'
STATUS_ROOM_HAS_ITEMS = 'roomhasitems'

class Status():
    def __init__(self, type, message=None):
        self.type = type
        self.message = message

    def get_dict(self) -> dict:
        """
        Converts the Status-object to a transportable dict
        :return: the dict
        """
        return {
            'status': self.type,
            'message': self.message
        }


#!/usr/bin/python
# -*- coding: utf-8 -*-

from Homevee.Item.ChatMessage import ChatMessage
from Homevee.Utils.Database_NEW import Database


def get_chat_messages(user, time, limit, db: Database = None):
    if db is None:
        db = Database()
    messages = ChatMessage.load_all_by_time(time, limit)
    return {'messages': ChatMessage.list_to_dict(messages)}

def get_chat_image(user, imageid, db: Database = None):
    if db is None:
        db = Database()
    return

def send_chat_message(user, data, db: Database = None):
    if db is None:
        db = Database()
    message = ChatMessage(user, data)
    return message.send()


from Homevee.Item.Logic.Action import Action
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.NotificationManager import NotificationManager


class PushNotificationAction(Action):
    def __init__(self, users, msg):
        super(PushNotificationAction, self).__init__()
        self.users = users
        self.msg = msg

    def run(self):
        NotificationManager().send_notification_to_users(self.users, "Homevee", self.msg, Database())

    @staticmethod
    def get_from_dict(dict):
        try:
            users = dict['users']
            msg = dict['msg']
            return PushNotificationAction(users, msg)
        except:
            return None


def get_system_info():
    systeminfo = []

    # Remote-ID
    #systeminfo.append({'name': 'Remote-ID', 'type': 'remote', 'value': db.get_server_data("REMOTE_ID", Database())})

    #Laufzeit
    uptime = "xy Stunden"
    systeminfo.append({'name': 'Laufzeit', 'type': 'uptime', 'value': uptime})

    #CPU-Temperatur
    cpu_temp = 35
    systeminfo.append({'name': 'CPU-Temp', 'type': 'cputemp', 'value': str(cpu_temp)+" &deg;"})

    #Kernel-Version
    kernel = "Version x.y"
    systeminfo.append({'name': 'Kernel', 'type': 'kernel', 'value': kernel})

    #Speichernutzung
    space_usage = "25%"
    systeminfo.append({'name': 'Speichernutzung', 'type': 'spaceusage', 'value': space_usage})

    #Freier Speicher
    free_space = "120Gb"
    systeminfo.append({'name': 'Freier Speicher', 'type': 'freespace', 'value': free_space})

    #RAM-Auslastung
    ram_usage = "37%"
    systeminfo.append({'name': 'RAM-Nutzung', 'type': 'ramusage', 'value': ram_usage})

    #Firmware
    firmware = "Firmware x.y.z"
    systeminfo.append({'name': 'Firmware', 'type': 'firmware', 'value': firmware})

    #Remote-ID
    systeminfo.append({'name': 'Firmware', 'type': 'firmware', 'value': firmware})

    return {'systeminfo': systeminfo}


from Homevee.Exception import AbstractFunctionCallException
from Homevee.Item.Logic.Action.ControlDeviceAction import ControlDeviceAction
from Homevee.Item.Logic.Action.PushNotificationAction import PushNotificationAction
from Homevee.Item.Logic.Action.RunSceneAction import RunSceneAction

ACTION_PUSH_NOTIFICATION = "push_notification"
ACTION_RUN_SCENE = "run_scene"
ACTION_CONTROL_DEVICE = "control_device"

class Action():
    def __init__(self):
        return

    def run(self):
        """
        Runs this action
        :return:
        """
        raise AbstractFunctionCallException("Action.run() is abstract")

    @staticmethod
    def get_module_map():
        module_map = {
            ACTION_PUSH_NOTIFICATION: PushNotificationAction,
            ACTION_RUN_SCENE: RunSceneAction,
            ACTION_CONTROL_DEVICE: ControlDeviceAction
        }
        return module_map


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.Item.Room import Room
from Homevee.Item.Status import *
from Homevee.Utils.Database_NEW import Database


def get_automations(user, room, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(room):
        return {'result': 'nopermission'}

    rules = []
    results = db.select_all("SELECT * FROM AUTOMATION_DATA WHERE LOCATION = :location", {'location': room})

    for item in results:
        data = get_full_automation_data(item['ID'], db)

        if 'result' in data and data['result'] == 'nopermission':
            continue

        rules.append(data)

    return {'rules': rules}

def get_full_automation_data(id, db: Database = None):
    if db is None:
        db = Database()
    data = db.select_all("SELECT * FROM AUTOMATION_DATA WHERE ID = :id", {'id': id})

    trigger_data = get_trigger_data(id, db)

    return {'name': data['NAME'], 'id': data['ID'], 'location': data['LOCATION'],
            'locationname': Room.get_name_by_id(data['LOCATION'], db), 'triggerdata': trigger_data,
            'conditiondata': data['CONDITION_DATA'], 'actiondata': data['ACTION_DATA'], 'isactive': True}

def get_trigger_data(id, db: Database = None):
    if db is None:
        db = Database()
        items = db.select_all("SELECT * FROM AUTOMATION_TRIGGER_DATA WHERE AUTOMATION_RULE_ID = :id", {'id': id})

        trigger_data = []

        for item in items:
            trigger_data.append({'type': item['TYPE'], 'id': item['ID'], 'value': item['VALUE'], 'text': item['TEXT']})

        return trigger_data

def add_edit_automation_rule(user, location, id, name, trigger_data, condition_data, action_data, is_active, db: Database = None):
    if db is None:
        db = Database()
    if not user.has_permission(location):
        return {'result': 'nopermission'}

    add_new = (id == None or id == "" or id == "-1")

    if(add_new):
        id = db.insert("INSERT INTO AUTOMATION_DATA (LOCATION, NAME, CONDITION_DATA, ACTION_DATA, IS_ACTIVE) VALUES (:location, :name, :conditions, :actions, :active)",
                    {'location': location, 'name': name, 'conditions': condition_data, 'actions': action_data, 'active': is_active})

        trigger_data = json.loads(trigger_data)

        add_trigger_data(trigger_data, id, db)
        return Status(type=STATUS_OK).get_dict()

    else:
        db.update("UPDATE AUTOMATION_DATA SET LOCATION = :location, NAME = :name, CONDITION_DATA = :conditions, ACTION_DATA = :actions, IS_ACTIVE = :active WHERE ID = :id",
            {'location': location, 'name': name, 'conditions': condition_data, 'actions': action_data, 'active': is_active, 'id': id})

        trigger_data = json.loads(trigger_data)

        db.delete("DELETE FROM AUTOMATION_TRIGGER_DATA WHERE AUTOMATION_RULE_ID = :id", {'id': id})

        add_trigger_data(trigger_data, id, db)
        return Status(type=STATUS_OK).get_dict()

def delete_automation_rule(user, id, db: Database = None):
    if db is None:
        db = Database()
    db.delete("DELETE FROM AUTOMATION_DATA WHERE ID = :id", {'id': id})
    db.delete("DELETE FROM AUTOMATION_TRIGGER_DATA WHERE AUTOMATION_RULE_ID = :id", {'id': id})

    return Status(type=STATUS_OK).get_dict()

def add_trigger_data(data, id, db: Database = None):
    if db is None:
        db = Database()
    for data in data:
        param_array = {'rule': id, 'type': data['type'], 'text': data['textdata']}

        if('id' in data):
            param_array['id'] = data['id']
        else:
            param_array['id'] = None

        if('value' in data):
            param_array['value'] = data['value']
        else:
            param_array['value'] = None

        db.insert("""INSERT INTO AUTOMATION_TRIGGER_DATA (AUTOMATION_RULE_ID, TYPE, ID, VALUE, TEXT) 
        VALUES (:rule, :type, :id, :value, :text)""", param_array)


from Homevee.Item.Logic.Action import Action


class Logic():
    def __init__(self):
        return

    @staticmethod
    def get_action_from_dict(dict):
        type = dict['action_type']

        module_map = Action.get_module_map()

        if type not in module_map:
            return None

        return module_map[type].get_from_dict(dict)

    @staticmethod
    def get_actions_from_dict(dict):
        actions = []

        for item in dict:
            action = Logic.get_action_from_dict(item)

            if action is not None:
                actions.append(action)

        return action


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant.Modules import VoiceModule


class VoiceGetTvScheduleModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            ['was',['kommt','ist','gibt','läuft'],['tv','fernsehen','fernseher']]
        ]

    def get_label(self):
        return "gettvschedule"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.get_tv(username, text, context, db)

    def get_tv(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return {'msg_speech':'TV-Programm', 'msg_text':'TV-Programm'}


class Response():
    def __init__(self, status_code, response):
        self.status_code = status_code
        self.response = response

    def get_dict(self):
        return {'status_code': self.status_code, 'response': self.response}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import socket
import time
import traceback
from time import sleep

import paho.mqtt.client as mqtt

from Homevee.DeviceAPI.mqtt_sensor import handle_mqtt_sensor
from Homevee.Helper import Logger, smarthome_functions, translations
from Homevee.Item.Gateway import *

MQTT_CLIENT = None

topics_to_subscribe = [
    '/home/#'
]

QUALITY_OF_SERVICE = 0

# https://www.dinotools.de/2015/04/12/mqtt-mit-python-nutzen/

def run_device_action(topic, msg, db: Database = None):
    if db is None:
        db = Database()
    topic_parts = topic.split("/")

    #topic_parts[0] is empty
    #topic_parts[1] is "home"

    result = None

    if(topic_parts[2] == "device"):
        result = db.select_all("SELECT * FROM MQTT_DEVICES WHERE TOPIC = :topic",
                    {'topic': topic})

        key = result['KEY']

        data = json.loads(msg)

        device_types = ["MQTT_SENSORS"]  # , "MQTT_TRIGGERS"]

        for device_type in device_types:
            results = db.select_all("SELECT * FROM " + device_type + " WHERE DEVICE_ID = :dev_id",
                        {'dev_id': result['ID']})

            for device in results:
                for data_item in data:
                    if str(device['VALUE_ID']) == str(data_item['id']):
                        handle_mqtt_sensor(device['ID'], data_item['value'])

        # Run MQTT-Trigger
        '''db.select_all("SELECT * FROM MQTT_TRIGGERS WHERE TOPIC = :topic", {'topic': message.topic})
        for item in cur.fetchall():
            #handle item action
            data = json.loads(msg)
            Logger.log(data)
            Logger.log(item)
            run_trigger_automation("MQTT-Trigger", item['TYPE'], item['ID'], data['action'])
            break'''
    elif(topic_parts[2] == "assistant" and topic_parts[4]=="send"):
        start_time = time.time()
        smart_speaker = db.select_all("SELECT * FROM SMART_SPEAKER WHERE ID = :id",
                    {'id': topic_parts[3]})

        topic = '/' + topic_parts[1] + '/' + topic_parts[2] + '/' + topic_parts[3] + '/receive'

        try:
            data = json.loads(msg)

            data = json.loads(data['msg'])

            username = data['username']
            text = data['text']

            text = text.encode('utf-8')

            result = smarthome_functions.do_voice_command(username, text, None, smart_speaker['LOCATION'], db, translations.LANGUAGE)

            result['time'] = time.time()
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            answer = "Es gab einen Fehler."
            result = {'msg_text': answer, 'msg_speech': answer}

        end_time = time.time()
        result['computing_duration'] = end_time - start_time

        publish(topic, json.dumps(result))

def publish(topic, msg):
    client = mqtt.Client()

    gateway = Item.load_from_db(Gateway, MQTT_BROKER, Database())

    Logger.log("publishing to "+topic+": "+msg)

    client.connect(gateway.ip)
    client.publish(topic, msg, QUALITY_OF_SERVICE, False)

def on_message(client, userdata, message):
    db = Database()

    msg = str(message.payload.decode("utf-8"))
    Logger.log("message received: ", msg)
    Logger.log("message topic: ", message.topic)
    # Logger.log("message qos=",message.qos)
    # Logger.log("message retain flag=",message.retain)

    # decrypt message with saved key of the device in db
    try:
        run_device_action(message.topic, msg)
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()

def on_connect(client, userdata, flags, rc):
    for topic in topics_to_subscribe:
        client.subscribe(topic, QUALITY_OF_SERVICE)

    for topic in get_topics():
        client.subscribe(topic, QUALITY_OF_SERVICE)


def init_client():
    while (True):
        gateway = Item.load_from_db(Gateway, MQTT_BROKER, Database())

        if gateway is None:
            sleep(30)

        try:
            broker_address = gateway.ip
        except:
            broker_address = None

        #keep retrying when broker_address is None
        if broker_address is None:
            sleep(15)
            continue

        try:
            client = mqtt.Client()
            client.on_connect = on_connect
            client.on_message = on_message

            client.connect(broker_address)

            Logger.log("Connected to MQTT Broker: " + broker_address)


            while(True):
                #check if client still connected and exit if not
                client.loop(1)

        except socket.error as e:
            continue
        # Logger.log("Cannot reach MQTT Broker: " + broker_address)


def get_topics():
    # load topics from database

    return []



from Homevee.Exception import AbstractFunctionCallException, InvalidParametersException
from Homevee.Item import Item
from Homevee.Item.Room import Room
from Homevee.Utils.Database_NEW import Database


class Device(Item):
    def __init__(self, name, icon, location, id=None):
        super(Device, self).__init__()

        self.id = id
        self.name = name
        self.icon = icon
        self.location = location

    def get_device_type(self) -> str:
        """
        Returns the devices type string
        :return: the device type
        """
        raise AbstractFunctionCallException("Device.get_device_type() is abstract")

    def get_dict(self, fields=None):
        output_dict = {}

        dict = self.build_dict()

        if (fields is None):
            output_dict = dict
        else:
            try:
                for field in fields:
                    output_dict[field] = dict[field]
            except:
                raise InvalidParametersException("InvalidParams given for get_dict()")

        output_dict['device_type'] = self.get_device_type()

        return output_dict

    @staticmethod
    def get_all(module, location=None, db: Database = None):
        if db is None:
            db = Database()
        devices = []

        if not isinstance(location, Room):
            location = Item.load_from_db(Room, location)

        #print("get all from location: "+location.name+" ("+str(location.id)+")")

        all_devices = module.load_all(db)

        if location is None or location == "all":
            # get all devices of all types
            return all_devices

        for device in all_devices:
            #print("location: "+str(device.location)+" == "+str(location.id))
            if str(device.location) == str(location.id):
                #print("append...")
                print(device.get_dict())
                devices.append(device)

        return devices


#!/usr/bin/python
# -*- coding: utf-8 -*-

from Homevee.Utils.Database_NEW import Database


class VoiceModule():
    def __init__(self, priority=1):
        self.priority = priority

    def get_context_key(self) -> int:
        """
        Get the context key
        :return: the context key
        """
        raise None

    def get_priority(self) -> int:
        """
        Get the priority
        :return: the priority
        """
        return self.priority

    def get_pattern(self, db: Database) -> list:
        """
        Get the pattern
        :param db: the database connection
        :return: the pattern
        """
        return None

    def get_label(self) -> str:
        """
        Get the label
        :return: the label
        """
        return None

    def run_command(self, username: str, text: str, context: dict, db: Database = None):
        if db is None:
            db = Database()
        """
        Run the voice command
        :param username: the username of the calling user
        :param text: the voice command text
        :param context: the context data dict
        :param db: the database connection
        :return:
        """
        pass

    #helper functions
    def is_number(self, s: str) -> bool:
        """
        Checks if the given string is a number
        :param s: the string
        :return: true if the string is a number, false otherwise
        """
        try:
            float(s)
            return True
        except ValueError:
            return False


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import argparse

from Homevee import Homevee
from Homevee.Helper import Logger
from Homevee.Helper.translations import translate_print, translate
from Homevee.Manager.user import has_users
from Homevee.Utils.Database_NEW import Database


def main():
    homevee = Homevee()

    #parse args
    parser = argparse.ArgumentParser(description='Homevee ist dein neues Smarthome-System!')
    parser.add_argument('--mode', default="start", type=str, help='Modus; default: Homevee starten (start)')
    parser.add_argument('--use_cloud', default=True, type=bool, help='Gibt an, ob die Cloud-Verbindung genutzt werden soll')
    parser.add_argument('--websocket_server', default=False, type=bool, help='Gibt an, ob der Websocket-Server genutzt werden soll')
    parser.add_argument('--http_server', default=False, type=bool, help='Gibt an, ob HTTP-Server genutzt werden soll')
    parser.add_argument('--is_admin', default=False, type=bool, help='Gibt an, ob der neu hinzugefügte Nutzer ein Administrator sein soll')
    parser.add_argument('--is_debug', default=False, type=bool, help='Gibt an, ob Debug-Meldungen ausgegeben werden sollen')

    args = parser.parse_args()

    Logger.IS_DEBUG = args.is_debug

    #print(args)

    #check if Homevee has atleast 1 user
    if not has_users(Database()):
        translate_print("no_users_create_admin")
        add_user(homevee, is_admin=True)

    if args.mode == "start":
        start_server(homevee)
    elif args.mode == "add_user":
        add_user(homevee, args.is_admin)

def start_server(homevee):
    homevee.start()

def get_password():
    return input(translate('enter_password'))

    password = getpass()  # prompt="Gib ein Passwort ein:")
    #print(password)

    password_again = getpass()  # prompt="Wiederhole das Passwort:")
    #print(password_again)

    while (password != password_again):
        translate_print('passwords_dont_match')
        password = getpass()  # prompt="Gib ein Passwort ein:")
        password_again = getpass()  # prompt="Wiederhole das Passwort:")

    return password

def add_user(homevee, is_admin=False):
    username = input(translate('enter_username'))
    #print(username)

    password = get_password()

    homevee.add_user(username=username, password=password, is_admin=is_admin)


if __name__ == "__main__":
    main()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import socket
import traceback
from _thread import start_new_thread

from Homevee.Helper import Logger

SPEAKER_HOST = ''
SPEAKER_PORT = 7888

class SmartSpeakerConnection():
    def __init__(self):
        pass

    def start_loop(self):
        start_new_thread(self.smart_speaker_loop, ())

    def smart_speaker_loop(self):
        while(True):
            self.start_server_socket()
        return

    def clientthread(self, conn):
        return


    def listen_for_requests(self, s):
        # wait to accept a connection - blocking call
        conn, addr = s.accept()
        Logger.log(('Connected with ' + addr[0] + ':' + str(addr[1])))

        # start new thread takes 1st argument as a function name to be run, second is the tuple of arguments to the function.
        start_new_thread(self.clientthread, (conn,))

    def start_server_socket(self):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        Logger.log('Socket created')

        # send_notification_to_users(['sascha'], "Titel", "Nachricht", Database())

        # Bind socket to local host and port
        try:
            s.bind((SPEAKER_HOST, SPEAKER_PORT))
        except socket.error as msg:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            Logger.log(('Bind failed. Error Code : ' + str(msg[0]) + ' Message ' + msg[1]))

        Logger.log('Socket bind complete')

        # Start listening on socket
        s.listen(10)
        Logger.log('Socket now listening')

        while 1:
            try:
                self.listen_for_requests(s)
            except:
                if(Logger.IS_DEBUG):
                    traceback.print_exc()
                break

        s.close()


from Homevee.Utils.Database_NEW import Database
from Homevee.VoiceAssistant.Modules import VoiceModule


class VoiceShoppingListModule(VoiceModule):
    def find_items(self, text, db: Database = None):
        if db is None:
            db = Database()
        items = []

        results = db.select_all("SELECT * FROM SHOPPING_LIST", {})

        for item in results:
            position = text.find(item['ITEM'].lower())
            if position is not -1:
                items.append({'item': item['ITEM'], 'amount': item['AMOUNT'], 'id': item['ID']})

        return items


from Homevee.Exception import DatabaseSaveFailedException, InvalidParametersException
from Homevee.Item import Item
from Homevee.Utils.Database_NEW import Database


class Person(Item):
    def __init__(self, name, nickname, phone_number, address, birthdate, longitude, latitude, id=None):
        super(Person, self).__init__()
        self.name = name
        self.nickname = nickname
        self.phone_number = phone_number
        self.address = address
        self.birthdate = birthdate
        self.longitude = longitude
        self.latitude = latitude
        self.id = id

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.delete("DELETE FROM PEOPLE_DATA WHERE ID == :id", {'id': self.id})
            return True
        except:
            return False

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            if (self.id is None or self.id == ""):
                new_id = db.insert("""INSERT INTO PEOPLE_DATA (NAME, NICKNAME, PHONE_NUMBER, ADDRESS, BIRTHDATE, LONGITUDE, 
                LATITUDE) VALUES (:name, :nickname, :phone_number, :address, :birthdate, :longitude, :latitude)""",
                            {'name': self.name, 'nickname': self.nickname, 'phone_number': self.phone_number,
                             'address': self.address, 'birthdate': self.birthdate, 'longitude': self.longitude,
                             'latitude': self.latitude})
                self.id = new_id

            # update
            else:
                db.update("""UPDATE PEOPLE_DATA SET NAME = :name, NICKNAME = :nickname, PHONE_NUMBER = :phone_number,
                 ADDRESS = :address, BIRHTDATE = :birthdate, LONGITUDE = :longitude, LATITUDE = :latitude WHERE ID = :id""",
                                {'name': self.name, 'nickname': self.nickname, 'phone_number': self.phone_number,
                                 'address': self.address, 'birthdate': self.birthdate, 'longitude': self.longitude,
                                 'latitude': self.latitude, 'id': self.id})
                #TODO add generated id to object
        except:
            raise DatabaseSaveFailedException("Could not save event to database")

    def build_dict(self):
        dict = {
            'id': self.id,
            'name': self.name,
            'nickname': self.nickname,
            'phone_number': self.phone_number,
            'address': self.address,
            'birthdate': self.birthdate,
            'longitude': self.longitude,
            'latitude': self.latitude
        }
        return dict

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = Person(result['NAME'], result['NICKNAME'], result['PHONE_NUMBER'], result['ADDRESS'], result['BIRTHDATE'], result['LONGITUDE'], result['LATITUDE'], result['ID'])
            items.append(item)
        return items

    @staticmethod
    def load_all_ids_from_db(ids, db: Database = None):
        if db is None:
            db = Database()
        return Person.load_all_from_db('SELECT * FROM PEROPLE_DATA WHERE ID IN (%s)'
                                              % ','.join('?' * len(ids)), ids, db)

    @staticmethod
    def load_all(db: Database):
        return Person.load_all_from_db('SELECT * FROM PEOPLE_DATA', {}, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            name = dict['name']
            nickname = dict['nickname']
            phone_number = dict['phone_number']
            address = dict['address']
            birthdate = dict['birthdate']
            latitude = dict['latitude']
            longitude = dict['longitude']

            item = Person(name, nickname, phone_number, address, birthdate, longitude, latitude, id)

            return item
        except:
            raise InvalidParametersException("Person.create_from_dict(): invalid dict")


import json

from Homevee.Item import Item
from Homevee.Item.Room import Room


class Permission(Item):
    def __init__(self, permission_key, name=None):
        super(Permission, self).__init__()

        if name is not None:
            self.name = name
        else:
            if permission_key == "admin":
                self.name = "Administrator"
            else:
                self.name = Room.get_name_by_id(permission_key, None)

        self.key = permission_key

    def build_dict(self):
        dict = {
            'key': self.key,
            'name': self.name
        }
        return dict

    @staticmethod
    def create_list_from_json(json_string: str) -> list:
        """
        Creates a list of Permission from a
        json-string of the format {"permissions":[...]}
        :param json_string: the json string
        :return: the list of permissions
        """
        data = json.loads(json_string)
        permissions = []
        for item in data['permissions']:
            permissions.append(Permission(item))
        return permissions

    @staticmethod
    def list_to_dict(list: list) -> list:
        """
        Converts a list of Permission to a list of dicts
        :param list: the list of items to convert
        :return: the list of dicts
        """
        permissions = []
        for item in list:
            permissions.append(item.get_dict())
        return permissions

    @staticmethod
    def list_from_dict(dicts: dict) -> list:
        """
        Creates a list of permissions from a list of dicts
        :param dict: the list of dicts of permissions
        :return: the list of permissions
        """
        permissions = []
        for item in dicts:
            permissions.append(Permission(item['key'], item['name']))
        return permissions


    @staticmethod
    def get_json_list(permissions: list) -> dict:
        """
        Creates a json-list-string from a list of permissions
        :param permissions: list of permissions
        :return: the json-string
        """
        data = []
        for permission in permissions:
            data.append(permission.key)
        permission_data = {'permissions': data}
        return json.dumps(permission_data)


#!/usr/bin/python
# -*- coding: utf-8 -*-

from Homevee.Cronjob import IntervalCronjob
from Homevee.DeviceAPI.zwave.get_devices import set_device_value
from Homevee.DeviceAPI.zwave.utils import do_zwave_request
from Homevee.Utils.Database_NEW import Database
from Homevee.Utils.DeviceTypes import *


class ZWaveValueLoaderCronjob(IntervalCronjob):
    def __init__(self):
        super(ZWaveValueLoaderCronjob, self).__init__(task_name="ZWaveValueLoaderCronjob", interval_seconds=5*60)

    def task_to_do(self, *args):
        self.load_zwave_values()

    def load_zwave_values(self):
        db = Database()



        # Alle Z-Wave Gerätetypen durchlaufen und den aktuellen Wert in Datenbank schreiben

        #Stromzähler
        TYPE = ZWAVE_POWER_METER
        data = db.select_all("SELECT * FROM ZWAVE_POWER_METER", {})

        for item in data:
            ID = item['DEVICE_ID']
            result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID)

            if result is None or result['code'] != 200:
                value = "N/A"
            else:
                value = result['data']['metrics']['level']

            set_device_value(TYPE, ID, value, db)

        #Thermostat
        TYPE = ZWAVE_THERMOSTAT
        data = db.select_all("SELECT * FROM ZWAVE_THERMOSTATS", {})

        for item in data:
            ID = item['THERMOSTAT_ID']
            result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID)

            if result is None or result['code'] != 200:
                value = "N/A"
            else:
                value = result['data']['metrics']['level']

            set_device_value(TYPE, ID, value, db)

        #Sensor
        TYPE = ZWAVE_SENSOR
        data = db.select_all("SELECT * FROM ZWAVE_SENSOREN", {})

        for item in data:
            ID = item['ID']
            result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID)

            if result is None or result['code'] != 200:
                value = "N/A"
            else:
                value = result['data']['metrics']['level']

                #if item['VALUE'] != value:
                    #trigger automation

            set_device_value(TYPE, ID, value, db)

        #Schalter
        TYPE = ZWAVE_SWITCH
        data = db.select_all("SELECT * FROM ZWAVE_SWITCHES", {})

        for item in data:
            ID = item['ID']
            result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID)

            if result is None or result['code'] != 200:
                value = "N/A"
            else:
                value = result['data']['metrics']['level']

                value = value == "on"

            set_device_value(TYPE, ID, value, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from Homevee.Helper import Logger
from Homevee.Manager.custom_voice_commands import run_custom_voice_commands
from Homevee.VoiceAssistant import Helper
from Homevee.VoiceAssistant.Modules.ConversationModule import VoiceConversationModule
# reload(sys)
# sys.setdefaultencoding('utf8')
from Homevee.VoiceAssistant.VoiceModules import get_voice_modules


class VoiceAssistant():
    def do_voice_command(self, username, text, user_last_location, room, db, language):
        return self.voice_command(username, text, user_last_location, room, db, language)

    def voice_command_cloud(self, username, text, user_last_location, room, db, language):
        text = text.lower()

        text = Helper.replace_voice_commands(text, username, db)

        # run custom voice_commands
        answer = run_custom_voice_commands(text, username, db)

        if (answer is not None):
            return {'msg_speech': answer, 'msg_text': answer}

        Logger.log("Voice command: " + text)

        context = Helper.get_context(username, db)

        voice_modules = get_voice_modules()

        if context is not None:
            return self.run_context(voice_modules, context, username, text, user_last_location, room, db, language)
        else:
            MIN_CONFIDENCE = 0.95

            try:
                result = Helper.classify_voice_command(text)
            except:
                return self.do_voice_command(username, text, user_last_location, room, db, language)

            Logger.log(result)

            label = result['label']
            confidence = result['confidence']

            module_map = {}

            for module in voice_modules:
                module_map[module.get_label()] = module.run_command

            if confidence >= MIN_CONFIDENCE and label in module_map:
                answer = module_map[label](username, text, context, db)
            else:
                answer = VoiceConversationModule().run_command(username, text, context, db)

        return (answer)

    def voice_command(self, username, text, user_last_location, room, db, language):
        #return {'msg_speech': "Der Assistent ist noch nicht erreichbar.", 'msg_text': "Der Assistent ist zur Zeit noch nicht fertig, wird aber gerade implementiert."}

        #room => damit z.b. bei "mach das licht an" das licht im richtigen raum angeht

        #Logger.log "voice_command"

        text = text.lower()

        #use ai-classifier in cloud
        #try:
        #    helper.classify_voice_command(text)
        #except:
        #    Logger.log("Homevee assistant not working")

        text = Helper.replace_voice_commands(text, username, db)

        #run custom voice_commands
        answer = run_custom_voice_commands(text, username, db)

        if(answer is not None):
            return {'msg_speech': answer, 'msg_text': answer}

        #Logger.log("Voice command: "+text)

        context = Helper.get_context(username, db)

        voice_modules = get_voice_modules()

        if context is not None:
            return self.run_context(voice_modules, context, username, text, user_last_location, room, db, language)
        else:
            for module in voice_modules:
                if(Helper.contains_pattern(module.get_pattern(db), text)):
                    answer = module.run_command(username, text, context, db)
                    break

            if answer is None:
                answer = VoiceConversationModule().run_command(username, text, context, db)

        Logger.log(answer)

        return answer

    def run_context(self, voice_modules, context, username, text, user_last_location, room, db, language):
        answer = None

        context_data = json.loads(context['CONTEXT_DATA'])

        Logger.log("Kontext: " + str(context_data))

        for module in voice_modules:
            context_key = module.get_context_key()

            if context_key is None:
                continue

            if isinstance(context_key, list):
                if context['CONTEXT_KEY'] in context_key:
                    answer = module.run_command(username, text, context_data, db)
            else:
                if context['CONTEXT_KEY'] == context_key:
                    answer = module.run_command(username, text, context_data, db)

        if answer is None or not answer:
            return self.do_voice_command(username, text, user_last_location, room, db, language)

        return (answer)


import traceback

from Homevee.Exception import DatabaseSaveFailedException, InvalidParametersException
from Homevee.Helper import Logger
from Homevee.Item import Item, User
from Homevee.Utils.Database_NEW import Database


class Event(Item):
    def __init__(self, text, type, timestamp=None, id=None):
        super(Event, self).__init__()

        self.timestamp = timestamp
        self.text = text
        self.type = type
        self.id = id

    def delete(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            db.delete("DELETE FROM EVENTS WHERE ID == :id", {'id': self.id})
            return True
        except:
            return False

    def save_to_db(self, db: Database = None):
        if db is None:
            db = Database()
        try:
            if (self.id is None or self.id == ""):
                db.insert("""INSERT INTO EVENTS (TEXT, TYPE) VALUES (:text, :type)""",
                            {'text': self.text, 'type': self.type})
            # update
            else:
                db.update("""UPDATE EVENTS SET TEXT = :text, TYPE = :type WHERE ID = :id""",
                            {'text': self.text, 'type': self.type, 'id': self.id})
                #TODO add generated id to object
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save event to database")

    def build_dict(self):
        dict = {
            'id': self.id,
            'timestamp': self.timestamp,
            'text': self.text,
            'type': self.type
        }
        return dict

    @staticmethod
    def load_all_from_db(query, params, db: Database = None):
        if db is None:
            db = Database()
        if db is None:
            db = Database()
        items = []
        for result in db.select_all(query, params):
            item = Event(result['TEXT'], result['TYPE'], result['TIMESTAMP'], result['ID'])
            items.append(item)
        return items

    @staticmethod
    def load_all_from_db_desc_date_by_type(offset: int, limit: int, type: str = None, db: Database = None) -> list:
        """
        Loads all events by the given type ordered by descending timestamp
        :param offset: skip the first x results
        :param limit: only show x results
        :param type: the type of the events to show
        :param db: the database connection
        :return: list of events
        """
        params = {'limit': limit, 'offset': offset}
        where_clause = ""
        if type is not None and type != "":
            params['type'] = type
            where_clause = "WHERE TYPE == :type "
        query = "SELECT * FROM 'EVENTS' " + where_clause + "ORDER BY TIMESTAMP DESC LIMIT :limit OFFSET :offset"
        return Event.load_all_from_db(query, params)

    @staticmethod
    def get_unseen_events(user: User, db: Database = None) -> int:
        """
        Returns the number of unseen events for the given user
        :param user: the user
        :param db: the database connection
        :return: number of unseen events
        """
        last_checked = user.events_last_checked
        events = Event.load_all_from_db("SELECT * FROM 'EVENTS' WHERE TIMESTAMP > :time",
                        {'time': last_checked})

        return len(events)

    @staticmethod
    def load_all_types_from_db(types, db: Database = None):
        if db is None:
            db = Database()
        query = 'SELECT * FROM EVENTS WHERE TYPE IN (%s)' % ','.join('?'*len(types)), types
        return Event.load_all_from_db(query, {})

    @staticmethod
    def load_all(db: Database):
        return Event.load_all_from_db('SELECT * FROM EVENTS', {})

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            timestamp = dict['timestamp']
            text = dict['text']
            type = dict['type']

            item = Event(text, type, timestamp, id)

            return item
        except:
            raise InvalidParametersException("Event.create_from_dict(): invalid dict")


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib.error
import urllib.parse
import urllib.request

from Homevee.DeviceAPI import kodi_api
from Homevee.Helper import Logger
from Homevee.Utils.Database_NEW import Database

TYPE_KODI = "KODI"

def get_media_centers(user, db: Database = None):
    if db is None:
        db = Database()
    media_centers = []

    results = db.select_all("SELECT *, MEDIA_CENTER.NAME as MEDIA_CENTER_NAME, ROOMS.NAME as ROOM_NAME FROM MEDIA_CENTER, ROOMS WHERE ROOMS.LOCATION = MEDIA_CENTER.LOCATION", {})

    for result in results:
        if(user.has_permission(result['LOCATION'])):
            item = {'name': result['MEDIA_CENTER_NAME'], 'type': result['TYPE'], 'id': result['ID'],
                    'location': result['LOCATION'], 'value': result['ROOM_NAME']}
            media_centers.append(item)

    return {'mediacenters': media_centers}

def media_remote_action(type, id, remoteaction, db: Database = None):
    if db is None:
        db = Database()
    if type == TYPE_KODI:
        return kodi_api.media_remote_action(get_media_center_data(id), remoteaction)

def media_center_send_text(type, id, text, db: Database = None):
    if db is None:
        db = Database()
    if type == TYPE_KODI:
        return kodi_api.send_text(get_media_center_data(id), text)

    return "nosuchtype"

def get_media_center_music(type, id, limit, offset, db: Database = None):
    if db is None:
        db = Database()
    if type == TYPE_KODI:
        result = kodi_api.get_music(get_media_center_data(id), limit, offset)

        result = json.loads(result)

        songs = []

        song_array = result['result']['songs']

        for song in song_array:
            item = {'title': song['label'], 'artists': song['artist'], 'id': song['songid'], 'album': song['album']}
            songs.append(item)

        return {'songs': songs}

    return 'nosuchtype'

def get_media_center_artists(type, id, limit, offset, db: Database = None):
    if db is None:
        db = Database()
    if type == TYPE_KODI:
        result = kodi_api.get_artists(get_media_center_data(id), limit, offset)

        result = json.loads(result)

        artists = []

        artist_array = result['result']['artists']

        for artist in artist_array:
            item = {'artist': artist['artist'], 'id': artist['artistid'], 'thumbnail': artist['thumbnail']}
            artists.append(item)

        return {'artists': artists}

    return 'nosuchtype'

def get_media_center_albums(type, id, limit, offset, db: Database = None):
    if db is None:
        db = Database()
    if type == TYPE_KODI:
        result = kodi_api.get_albums(get_media_center_data(id), limit, offset)

        result = json.loads(result)

        albums = []

        album_array = result['result']['albums']

        for album in album_array:
            item = {'title': album['label'], 'artists': album['artist'], 'id': album['albumid'], 'thumbnail': album['thumbnail']}
            albums.append(item)

        return {'albums': albums}

    return 'nosuchtype'

def get_media_center_music_genres(type, id, limit, offset, db: Database = None):
    if db is None:
        db = Database()
    if type == TYPE_KODI:
        result = kodi_api.get_music_genres(get_media_center_data(id), limit, offset)

        result = json.loads(result)

        genres = []

        genre_array = result['result']['genres']

        for genre in genre_array:
            item = {'title': genre['label'], 'id': genre['genreid'], 'thumbnail': genre['thumbnail']}
            genres.append(item)

        return {'genres': genres}

    return 'nosuchtype'

def get_media_center_movie_genres(type, id, limit, offset, db: Database = None):
    if db is None:
        db = Database()
    if type == TYPE_KODI:
        result = kodi_api.get_movie_genres(get_media_center_data(id), limit, offset)

        Logger.log(result)

        result = json.loads(result)

        genres = []

        genre_array = result['result']['genres']

        for genre in genre_array:
            item = {'title': genre['label'], 'id': genre['genreid'], 'thumbnail': genre['thumbnail']}
            genres.append(item)

        return {'genres': genres}

    return 'nosuchtype'

def get_media_center_shows(type, id, limit, offset, db: Database = None):
    if db is None:
        db = Database()
    if type == TYPE_KODI:
        result = kodi_api.get_tv_show(get_media_center_data(id), limit, offset)

        result = json.loads(result)

        tvshows = []

        tvshow_array = result['result']['tvshows']

        for tvshow in tvshow_array:

            thumbnail = tvshow['thumbnail']
            thumbnail = thumbnail.replace('image://', '')
            thumbnail = urllib.parse.unquote(thumbnail).decode('utf8')

            item = {'title': tvshow['label'], 'genres': tvshow['genre'], 'id': tvshow['tvshowid'],
                    'year': ['year'], 'thumbnail': thumbnail}
            tvshows.append(item)

        return {'shows': tvshows}

    return 'nosuchtype'

def get_media_center_show_seasons(type, id, limit, offset, showid, db: Database = None):
    if db is None:
        db = Database()
    if type == TYPE_KODI:
        result = kodi_api.get_tv_show_seasons(get_media_center_data(id), showid, limit, offset)

        result = json.loads(result)

        seasons = []

        season_array = result['result']['tvshows']

        for season in season_array:

            thumbnail = season['thumbnail']
            thumbnail = thumbnail.replace('image://', '')
            thumbnail = urllib.parse.unquote(thumbnail).decode('utf8')

            item = {'title': season['label'], 'id': season['seasonid'], 'thumbnail': thumbnail}
            seasons.append(item)

        return {'seasons': seasons}

    return 'nosuchtype'

def get_media_center_show_episodes(type, id, limit, offset, showid, seasonid, db: Database = None):
    if db is None:
        db = Database()
    return []

def get_media_center_movies(type, id, limit, offset, db: Database = None):
    if db is None:
        db = Database()
    if type == TYPE_KODI:
        result = kodi_api.get_movies(get_media_center_data(id), limit, offset)

        result = json.loads(result)

        movies = []

        movie_array = result['result']['movies']

        for movie in movie_array:

            thumbnail = movie['thumbnail']
            thumbnail = thumbnail.replace('image://', '')
            thumbnail = urllib.parse.unquote(thumbnail).decode('utf8')

            item = {'title': movie['label'], 'genres': movie['genre'], 'id': movie['movieid'],
                    'year': movie['year'], 'thumbnail': thumbnail}
            movies.append(item)

        return {'movies': movies}

    return 'nosuchtype'

def get_media_center_playing(type, id, db: Database = None):
    if db is None:
        db = Database()
    return []

def get_media_menter_playing(type, id, db: Database = None):
    if db is None:
        db = Database()
    return []

def get_media_center_data(id, db: Database = None):
    if db is None:
        db = Database()

        cur = db.cursor()

        db.select_all("SELECT * FROM MEDIA_CENTER WHERE ID == :id", {'id': id})

        result = cur.fetchone()

        return result


from Homevee.VoiceAssistant.Modules.AddNutritionDataModule import VoiceAddNutritionDataModule
from Homevee.VoiceAssistant.Modules.CalculatorModule import VoiceCalculatorModule
from Homevee.VoiceAssistant.Modules.CalendarModule.AddCalendarModule import VoiceAddCalendarModule
from Homevee.VoiceAssistant.Modules.CalendarModule.GetCalendarModule import VoiceGetCalendarModule
from Homevee.VoiceAssistant.Modules.ConversationModule import VoiceConversationModule
from Homevee.VoiceAssistant.Modules.DeviceControlModule.GetSensorDataModule import VoiceDeviceGetSensorDataModule
from Homevee.VoiceAssistant.Modules.DeviceControlModule.RGBControlModule import VoiceRgbDeviceControlModule
from Homevee.VoiceAssistant.Modules.DeviceControlModule.SetModesModule import VoiceDeviceSetModesModule
from Homevee.VoiceAssistant.Modules.GetActivitiesModule import VoiceGetActivitiesModule
from Homevee.VoiceAssistant.Modules.GetJokesModule import VoiceGetJokesModule
from Homevee.VoiceAssistant.Modules.GetNutritionDiaryModule import VoiceGetNutritionDiaryModule
from Homevee.VoiceAssistant.Modules.GetNutritionInfoModule import VoiceGetNutritionInfoModule
from Homevee.VoiceAssistant.Modules.GetRecipesModule import VoiceGetRecipesModule
from Homevee.VoiceAssistant.Modules.GetSummaryModule import VoiceGetSummaryModule
from Homevee.VoiceAssistant.Modules.GetTVScheduleModule import VoiceGetTvScheduleModule
from Homevee.VoiceAssistant.Modules.GetTVTippsModule import VoiceGetTvTippsModule
from Homevee.VoiceAssistant.Modules.GetWeatherModule import VoiceGetWeatherModule
from Homevee.VoiceAssistant.Modules.GetWeekdayModule import VoiceGetWeekdayModule
from Homevee.VoiceAssistant.Modules.MovieApiModule import VoiceMovieApiModule
from Homevee.VoiceAssistant.Modules.PlacesApiModule import VoicePlacesApiModule
from Homevee.VoiceAssistant.Modules.ShoppingListModule.AddToShoppingListModule import \
    VoiceAddToShoppingListModule
from Homevee.VoiceAssistant.Modules.ShoppingListModule.GetShoppingListModule import VoiceGetShoppingListModule
from Homevee.VoiceAssistant.Modules.WikipediaDefinitionModule import VoiceGetWikipediaDefinitionModule


def get_voice_modules():
    voice_modules = [
        VoiceAddCalendarModule(priority=1),
        VoiceGetCalendarModule(priority=1),
        VoiceAddNutritionDataModule(priority=1),
        VoiceGetActivitiesModule(priority=1),
        VoiceGetNutritionInfoModule(priority=1),
        VoiceGetRecipesModule(priority=1),
        VoiceDeviceGetSensorDataModule(priority=1),
        VoiceGetSummaryModule(priority=1),
        VoiceGetTvScheduleModule(priority=1),
        VoiceGetTvTippsModule(priority=1),
        VoiceGetWeatherModule(priority=1),
        VoiceGetWeekdayModule(priority=1),
        VoiceGetWikipediaDefinitionModule(priority=1),
        VoiceGetJokesModule(priority=1),
        VoiceMovieApiModule(priority=1),
        VoiceGetNutritionDiaryModule(priority=1),
        VoicePlacesApiModule(priority=1),
        VoiceRgbDeviceControlModule(priority=1),
        VoiceDeviceSetModesModule(priority=1),
        VoiceAddToShoppingListModule(priority=1),
        VoiceGetShoppingListModule(priority=1),
        VoiceCalculatorModule(priority=1),
        VoiceConversationModule(priority=0),
        #VoiceModule(priority=1),
    ]

    #voice_modules = sorted(voice_modules, key=lambda x: x.get_priority(), reverse=True)

    return voice_modules